#include "Player.h"
#include <iostream>
#include <glm\glm.hpp>
#include <glm\gtc\matrix_transform.hpp>
#include <glm\gtc\type_ptr.hpp>
#include "../PhysxManager.h"

using namespace babyOnWheels;
using namespace scene;
using namespace entities;
using namespace physx;

glm::vec3 GetRotation(glm::mat4 matr);

Player::Player(SceneObject* _model, Shader* _shader)
	:Entity(_model, _shader)
{
	/*
		das leben wird zu beginn auf 100 gestzt
		punkte sind am anfang null
	*/
	health = 100;
	points = 0;
}

void Player::update(float time_delta) {
	//Update SceneObject
	
	glm::vec3 rot = GetRotation(model->modelMatrix);

	if (glm::degrees(rot.y) < 5 && glm::degrees(rot.y) > -5 &&
		glm::degrees(rot.z) < 5 && glm::degrees(rot.z) > -5) {
		//TODO: remove next lines after physix used
		if (!brake && speed < maxSpeed) {
			speed += 7 * time_delta;
		}
		//std::cout << speed << std::endl;

		//break and accelerate
		if (brake && speed > minSpeed) {
			speed -= breakStrength * time_delta;
		}
		if (accelerate && speed < maxSpeed) {
			speed += accelerateStrength * time_delta;
		}
		if (speed < minSpeed) speed = minSpeed;
		if (speed > maxSpeed) speed = maxSpeed;

		//Steer
		float bias = 0.2f;
		if (steer < 0 + bias && steer > 0 - bias) {
			steer = 0;
		}
		//Move
		model->body->collider->addTorque(PxVec3(0.0f, handling * steer * time_delta, 0.0f), PxForceMode::eVELOCITY_CHANGE);

		glm::vec3 rot = GetRotation(model->modelMatrix);

		//std::cout << rot.x << "," << rot.y << "," << rot.z << std::endl;
		//std::cout << steer << std::endl;
		float horizontalAngle = rot.x;
		float verticalAngle = 0;
		glm::vec3 direction(
			cos(verticalAngle) * sin(horizontalAngle),
			sin(verticalAngle),
			cos(verticalAngle) * cos(horizontalAngle)
			);
		model->body->collider->addForce(PxVec3(direction.x * time_delta * speed, 0, -direction.z * time_delta * speed), PxForceMode::eVELOCITY_CHANGE);
		model->body->collider->setLinearDamping(0.3f);
		model->body->collider->setAngularDamping(0.01f);
	}
	else if (glm::degrees(rot.y) > 89 || glm::degrees(rot.y) < -89 ||
		glm::degrees(rot.z) > 89 || glm::degrees(rot.z) < -89) {
		dead = true;
	}
	model->update(time_delta);
}

/*
METHODE ZUM LENKEN DES SPIELERS
*/
void Player::doSteer(float value) {
	//TODO: lenken
	steer = value;
}

void Player::jump() {
	PhysxManager::move(glm::vec3(0, -5, 0), model->body->collider);
}

/*
METHODE ZUM BESCHLEUNIGEN DES SPIELERS
*/
void Player::doAccelerate(bool value) {
	//TODO: beschleunigen
	accelerate = value;
}

/*
METHODE ZUM BREMSEN DES SPIELERS
*/
void Player::doBrake(bool value) {
	//TODO: bremsen
	brake = value;
}

Player::~Player()
{
}


glm::vec3 GetRotation(glm::mat4 matr)
{
	glm::vec3 rotation = glm::vec3(1);
	if (matr[0][0] == 1.0f)
	{
		rotation.x = atan2f(matr[0][2], matr[2][3]);
		rotation.y = 0;
		rotation.z = 0;

	}
	else if (matr[0][0] == -1.0f)
	{
		rotation.x = atan2f(matr[0][2], matr[2][3]);
		rotation.y = 0;
		rotation.z = 0;
	}
	else
	{

		rotation.x = atan2(-matr[2][0], matr[0][0]);
		rotation.y = asin(matr[1][0]);
		rotation.z = atan2(-matr[1][2], matr[1][1]);
	}
	return rotation;
}