#include "ParticleSystem.h"

#define MAX_PARTICLES 1000
#define PARTICLE_LIFETIME 10.0f

#define PARTICLE_TYPE_LAUNCHER 0.0f
#define PARTICLE_TYPE_SHELL 1.0f
#define PARTICLE_TYPE_SECONDARY_SHELL 2.0f

// #define COLOR_TEXTURE_UNIT              GL_TEXTURE0
// #define COLOR_TEXTURE_UNIT_INDEX        0
// #define RANDOM_TEXTURE_UNIT             GL_TEXTURE1
// #define RANDOM_TEXTURE_UNIT_INDEX       1

#define ZERO_MEM(a) memset(a, 0, sizeof(a))

struct Particle
{
	float Type;
	glm::vec3 Pos;
	glm::vec3 Vel;
	float LifetimeMillis;
};

ParticleSystem::ParticleSystem()
{
	m_currVB = 0;
	m_currTFB = 1;
	m_isFirst = true;
	m_time = 0;
	m_pTexture = NULL;

	ZERO_MEM(m_transformFeedback);
	ZERO_MEM(m_particleBuffer);
}


bool ParticleSystem::InitParticleSystem(glm::mat4* pos)
{
	Particle Particles[MAX_PARTICLES];
	ZERO_MEM(Particles);

	carriageMatrix = pos;

	glm::vec3 posVec;
	posVec.x = (*pos)[0][3];
	posVec.y = (*pos)[1][3];
	posVec.z = (*pos)[2][3];

	Particles[0].Type = PARTICLE_TYPE_LAUNCHER;
	Particles[0].Pos = posVec; // TODO
	Particles[0].Vel = glm::vec3(0.0f, 0.01f, 0.0f);
	Particles[0].LifetimeMillis = 0.0f;

	glGenTransformFeedbacks(2, m_transformFeedback);
	glGenBuffers(2, m_particleBuffer);

	for (unsigned int i = 0; i < 2; i++) {
		glBindTransformFeedback(GL_TRANSFORM_FEEDBACK, m_transformFeedback[i]);
		glBindBufferBase(GL_TRANSFORM_FEEDBACK_BUFFER, 0, m_particleBuffer[i]);
		glBindBuffer(GL_ARRAY_BUFFER, m_particleBuffer[i]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(Particles), Particles, GL_DYNAMIC_DRAW);
	}

	// initialize the update technique
	m_updateTechnique = LoadProgram("../Baby_On_Wheels/Shader/ps_update.vs", "../Baby_On_Wheels/Shader/ps_update.gs");

	m_deltaTimeMillisLocation = glGetUniformLocation(m_updateTechnique, "gDeltaTimeMillis");
	m_randomTextureLocation = glGetUniformLocation(m_updateTechnique, "gRandomTexture");
	m_timeLocation = glGetUniformLocation(m_updateTechnique, "gTime");
	m_launcherLifetimeLocation = glGetUniformLocation(m_updateTechnique, "gLauncherLifetime");
	m_shellLifetimeLocation = glGetUniformLocation(m_updateTechnique, "gShellLifetime");
	m_secondaryShellLifetimeLocation = glGetUniformLocation(m_updateTechnique, "gSecondaryShellLifetime");
	m_positionLauncherLocation = glGetUniformLocation(m_updateTechnique, "gPositionLauncher");
	glUniform1f(m_randomTextureLocation, 0);
	glUniform1f(m_launcherLifetimeLocation, 100.0f);
	glUniform1f(m_shellLifetimeLocation, 10000.0f);
	glUniform1f(m_secondaryShellLifetimeLocation, 2500.0f);

	
	m_randomTexture.InitRandomTexture(1000);
	if (!m_randomTexture.InitRandomTexture(1000)) {
		return false;
	}
	 
	m_randomTexture.Bind(GL_TEXTURE0);

	// m_billboardTechnique.SetBillboardSize(0.01f);

	m_billboardTechnique = LoadProgram("../Baby_On_Wheels/Shader/billboard_shader.vs", "../Baby_On_Wheels/Shader/billboard_shader.gs", "../Baby_On_Wheels/Shader/billboard_shader.fs");

	m_VPLocation = glGetUniformLocation(m_billboardTechnique, "gVP");
	m_cameraPosLocation = glGetUniformLocation(m_billboardTechnique, "gCameraPos");
	m_colorMapLocation = glGetUniformLocation(m_billboardTechnique, "gColorMap");

	glUniform1i(m_colorMapLocation, 0);

	m_pTexture = new babyOnWheels::Texture("../Baby_On_Wheels/Textures/street1.png");// eventuell Texture* wird zurckgegeben

	return true;

}


void ParticleSystem::Render(float DeltaTimeMillis, const glm::mat4 VP, const glm::vec3 CameraPos)
{
	m_time += DeltaTimeMillis;

	UpdateParticles(DeltaTimeMillis);

	RenderParticles(VP, CameraPos);

	m_currVB = m_currTFB;
	m_currTFB = (m_currTFB + 1) & 0x1;
}

void ParticleSystem::UpdateParticles(float DeltaTimeMillis)
{
	glUseProgram(m_updateTechnique);
	glUniform1f(m_timeLocation, m_time);
	glUniform1f(m_deltaTimeMillisLocation, DeltaTimeMillis);
	glUniform3f(m_positionLauncherLocation, (*carriageMatrix)[0][3], (*carriageMatrix)[1][3], (*carriageMatrix)[2][3]);
	// glUniform3f(m_positionLauncherLocation, 1.6, -1.1, -50.0);


	// std::cout << (*carriageMatrix)[3][0] << " " << (*carriageMatrix)[3][1] << " " << (*carriageMatrix)[3][2] << " " << (*carriageMatrix)[3][3] << std::endl;

	m_randomTexture.Bind(GL_TEXTURE0);

	glEnable(GL_RASTERIZER_DISCARD);

	glBindBuffer(GL_ARRAY_BUFFER, m_particleBuffer[m_currVB]);
	glBindTransformFeedback(GL_TRANSFORM_FEEDBACK, m_transformFeedback[m_currTFB]);

	glEnableVertexAttribArray(0);
	glEnableVertexAttribArray(1);
	glEnableVertexAttribArray(2);
	glEnableVertexAttribArray(3);

	glVertexAttribPointer(0, 1, GL_FLOAT, GL_FALSE, sizeof(Particle), 0);                          // type
	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, sizeof(Particle), (const GLvoid*)4);         // position
	glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, sizeof(Particle), (const GLvoid*)16);        // velocity
	glVertexAttribPointer(3, 1, GL_FLOAT, GL_FALSE, sizeof(Particle), (const GLvoid*)28);          // lifetime

	glBeginTransformFeedback(GL_POINTS);

	if (m_isFirst) {
		glDrawArrays(GL_POINTS, 0, 1);

		m_isFirst = false;
	}
	else {
		glDrawTransformFeedback(GL_POINTS, m_transformFeedback[m_currVB]);
	}

	glEndTransformFeedback();

	glDisableVertexAttribArray(0);
	glDisableVertexAttribArray(1);
	glDisableVertexAttribArray(2);
	glDisableVertexAttribArray(3);


}


void ParticleSystem::RenderParticles(const glm::mat4 VP, const glm::vec3 CameraPos)
{

	glUseProgram(m_billboardTechnique);
	glUniformMatrix4fv(m_VPLocation, 1, GL_TRUE, &VP[0][0]);
	glUniform3f(m_cameraPosLocation, CameraPos.x, CameraPos.y, CameraPos.z);

	m_pTexture->bind(GL_TEXTURE0);
	
	glDisable(GL_RASTERIZER_DISCARD);

	glBindBuffer(GL_ARRAY_BUFFER, m_particleBuffer[m_currTFB]);

	glEnableVertexAttribArray(0);

	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, sizeof(Particle), (const GLvoid*)4);  // position

	glDrawTransformFeedback(GL_POINTS, m_transformFeedback[m_currTFB]);

	glDisableVertexAttribArray(0);

}


ParticleSystem::~ParticleSystem()
{
	// SAFE_DELETE(m_pTexture);

	if (m_transformFeedback[0] != 0) {
		glDeleteTransformFeedbacks(2, m_transformFeedback);
	}

	if (m_particleBuffer[0] != 0) {
		glDeleteBuffers(2, m_particleBuffer);
	}
}



// used for particleEffects with geometry shader
GLuint ParticleSystem::LoadProgram(const std::string& vs, const std::string& gs, const std::string& fs)
{
	GLuint programHandle = glCreateProgram();

	// load vertex shader
	std::ifstream shaderFileVs(vs);
	std::string codeVs = std::string(
		std::istreambuf_iterator<char>(shaderFileVs),
		std::istreambuf_iterator<char>());
	shaderFileVs.close();

	const char * vsSource = codeVs.c_str();

	GLuint vsHandle = glCreateShader(GL_VERTEX_SHADER);
	glShaderSource(vsHandle, 1, &vsSource, nullptr);
	glCompileShader(vsHandle);
	glAttachShader(programHandle, vsHandle);

	// load fragment shader
	std::ifstream shaderFileFs(fs);
	std::string codeFs = std::string(
		std::istreambuf_iterator<char>(shaderFileFs),
		std::istreambuf_iterator<char>());
	shaderFileFs.close();
	const char * fsSource = codeFs.c_str();

	GLuint fsHandle = glCreateShader(GL_FRAGMENT_SHADER);
	glShaderSource(fsHandle, 1, &fsSource, nullptr);
	glCompileShader(fsHandle);
	glAttachShader(programHandle, fsHandle);

	// load geometry shader
	std::ifstream shaderFileGs(gs);
	std::string codeGs = std::string(
		std::istreambuf_iterator<char>(shaderFileGs),
		std::istreambuf_iterator<char>());
	shaderFileGs.close();
	const char * gsSource = codeGs.c_str();

	GLuint gsHandle = glCreateShader(GL_GEOMETRY_SHADER);
	glShaderSource(gsHandle, 1, &gsSource, nullptr);
	glCompileShader(gsHandle);
	glAttachShader(programHandle, gsHandle);

	glLinkProgram(programHandle);

	glUseProgram(programHandle);
	return programHandle;

}

// used for vertex + geometry shader
GLuint ParticleSystem::LoadProgram(const std::string& vs, const std::string& gs)
{
	GLuint programHandle = glCreateProgram();

	// load vertex shader
	std::ifstream shaderFileVs(vs);
	std::string codeVs = std::string(
		std::istreambuf_iterator<char>(shaderFileVs),
		std::istreambuf_iterator<char>());
	shaderFileVs.close();

	const char * vsSource = codeVs.c_str();

	GLuint vsHandle = glCreateShader(GL_VERTEX_SHADER);
	glShaderSource(vsHandle, 1, &vsSource, nullptr);
	glCompileShader(vsHandle);
	glAttachShader(programHandle, vsHandle);

	// load geometry shader
	std::ifstream shaderFileGs(gs);
	std::string codeGs = std::string(
		std::istreambuf_iterator<char>(shaderFileGs),
		std::istreambuf_iterator<char>());
	shaderFileGs.close();
	const char * gsSource = codeGs.c_str();

	GLuint gsHandle = glCreateShader(GL_GEOMETRY_SHADER);
	glShaderSource(gsHandle, 1, &gsSource, nullptr);
	glCompileShader(gsHandle);
	glAttachShader(programHandle, gsHandle);

	const GLchar* Varyings[4];
	Varyings[0] = "Type1"; // --> could be that the nale is wrong and the "1" has to be deleted
	Varyings[1] = "Position1";
	Varyings[2] = "Velocity1";
	Varyings[3] = "Age1";

	glTransformFeedbackVaryings(programHandle, 4, Varyings, GL_INTERLEAVED_ATTRIBS);

	glLinkProgram(programHandle);

	glUseProgram(programHandle);
	return programHandle;

}