#include "PhysxManager.h"
#include <iostream>
#include "glm\glm.hpp"


using namespace physx;
PxRigidStatic* groundPlane;

PhysxManager::PhysxManager()
{
	gFoundation = PxCreateFoundation(PX_PHYSICS_VERSION, gAllocator, gErrorCallback);
	PxProfileZoneManager* profileZoneManager = &PxProfileZoneManager::createProfileZoneManager(gFoundation);
	gPhysics = PxCreatePhysics(PX_PHYSICS_VERSION, *gFoundation, PxTolerancesScale(), true, profileZoneManager);


	PxSceneDesc sceneDesc(gPhysics->getTolerancesScale());
	sceneDesc.gravity = PxVec3(0.0f, 9.00f, 0.0f);
	gDispatcher = PxDefaultCpuDispatcherCreate(2);
	sceneDesc.cpuDispatcher = gDispatcher;
	sceneDesc.filterShader = PxDefaultSimulationFilterShader;
	gScene = gPhysics->createScene(sceneDesc);

	gMaterial = gPhysics->createMaterial(0.3f, 0.3f, 0.16f);

	//groundPlane = PxCreatePlane(*gPhysics, PxPlane(0, -1, 0, 2), *gMaterial);
	//gScene->addActor(*groundPlane);
}


PhysxManager::~PhysxManager()
{
}

void PhysxManager::update(float time_delta) {
	gScene->simulate(time_delta);
	gScene->fetchResults(true);

	const PxRenderBuffer& rb = gScene->getRenderBuffer();
	for (PxU32 i = 0; i < rb.getNbLines(); i++)
	{
		const PxDebugLine& line = rb.getLines()[i];
		// render the line
	}
}

Rigidbody* PhysxManager::createRigidbody(glm::mat4 modelMatrix, float mass, glm::vec3 size) {
	PxTransform pos = PxTransform(convertMatToPhysx(modelMatrix));
	PxShape* shape = gPhysics->createShape(PxBoxGeometry(size.x, size.y, size.z), *gMaterial);
	PxRigidDynamic* body = gPhysics->createRigidDynamic(pos);
	body->attachShape(*shape);
	PxRigidBodyExt::updateMassAndInertia(*body, mass);

	gScene->addActor(*body);
	shape->release();

	Rigidbody* rigid= new Rigidbody();
	rigid->collider = body;
	rigid->size = glm::vec3(size);

	return rigid;
}

ColliderStatic* PhysxManager::createStaticCollider(glm::mat4 modelMatrix, glm::vec3 size) {
	PxTransform pos = PxTransform(convertMatToPhysx(modelMatrix));
	PxShape* shape = gPhysics->createShape(PxBoxGeometry(size.x, size.y, size.z), *gMaterial);
	PxRigidStatic* body = gPhysics->createRigidStatic(pos);
	body->attachShape(*shape);
	gScene->addActor(*body);
	shape->release();
	ColliderStatic* coll = new ColliderStatic();
	coll->collider = body;
	coll->size = glm::vec3(size);

	return coll;
}

void PhysxManager::move(glm::vec3 velocity, PxRigidDynamic* body) {
	PxVec3 vel = PxVec3(velocity.x, velocity.y, velocity.z);
	body->setAngularDamping(0.5f);
	body->setLinearVelocity(vel);
}

glm::mat4 PhysxManager::getShapePosition(PxRigidDynamic* body) {
	return convertMat(PxMat44(body->getGlobalPose()));
}

void PhysxManager::cleanup() {
	gScene->release();
	gDispatcher->release();
	PxProfileZoneManager* profileZoneManager = gPhysics->getProfileZoneManager();
	gPhysics->release();
	profileZoneManager->release();
	gFoundation->release();
}

glm::mat4 PhysxManager::convertMat(const PxMat44 mat4)
{
	glm::mat4 newMat = glm::mat4(1);

	newMat[0][0] = mat4[0][0];
	newMat[0][1] = mat4[0][1];
	newMat[0][2] = mat4[0][2];
	newMat[0][3] = mat4[0][3];

	newMat[1][0] = mat4[1][0];
	newMat[1][1] = mat4[1][1];
	newMat[1][2] = mat4[1][2];
	newMat[1][3] = mat4[1][3];

	newMat[2][0] = mat4[2][0];
	newMat[2][1] = mat4[2][1];
	newMat[2][2] = mat4[2][2];
	newMat[2][3] = mat4[2][3];

	newMat[3][0] = mat4[3][0];
	newMat[3][1] = mat4[3][1];
	newMat[3][2] = mat4[3][2];
	newMat[3][3] = mat4[3][3];
	return newMat;
}

PxMat44 PhysxManager::convertMatToPhysx(const glm::mat4 mat4)
{
	PxMat44 newMat = PxMat44(1);

	newMat[0][0] = mat4[0][0];
	newMat[0][1] = mat4[0][1];
	newMat[0][2] = mat4[0][2];
	newMat[0][3] = mat4[0][3];

	newMat[1][0] = mat4[1][0];
	newMat[1][1] = mat4[1][1];
	newMat[1][2] = mat4[1][2];
	newMat[1][3] = mat4[1][3];

	newMat[2][0] = mat4[2][0];
	newMat[2][1] = mat4[2][1];
	newMat[2][2] = mat4[2][2];
	newMat[2][3] = mat4[2][3];

	newMat[3][0] = mat4[3][0];
	newMat[3][1] = mat4[3][1];
	newMat[3][2] = mat4[3][2];
	newMat[3][3] = mat4[3][3];
	return newMat;
}