#include "Physx/foundation/PxErrorCallback.h"
#include "Physx/PxPhysXConfig.h"
#include "Physx/PxPhysicsAPI.h"
#include "Physx/foundation/PxSimpleTypes.h"
#include <ctype.h>
#include "Physx\foundation\PxFoundation.h"
#include "glm\glm.hpp"

namespace physx {
	class UserErrorCallback : public PxErrorCallback
	{
	public:
		void reportError(PxErrorCode::Enum code, const char* message, const char* file,
			int line)
		{
			// error processing implementation
		}
	};
}

using namespace physx;

struct ColliderStatic
{
	PxRigidStatic* collider;
	glm::vec3 size;
};
struct Rigidbody
{
	PxRigidDynamic* collider;
	glm::vec3 size;
};

#pragma once
class PhysxManager
{
public:
	PhysxManager();
	~PhysxManager();

	void initPhysics();
	void cleanup();
	void update(float time_delta);
	Rigidbody* createRigidbody(glm::mat4 modelMatrix, float mass, glm::vec3 size);
	ColliderStatic* createStaticCollider(glm::mat4 modelMatrix, glm::vec3 size);
	static glm::mat4 getShapePosition(PxRigidDynamic* body);

	static void move(glm::vec3 velocity, PxRigidDynamic* body);
	static glm::mat4 convertMat(const PxMat44 mat4);
	static PxMat44 convertMatToPhysx(const glm::mat4 mat4);

	PxDefaultAllocator		gAllocator;
	UserErrorCallback	gErrorCallback;
	PxFoundation*			gFoundation = NULL;
	PxPhysics*				gPhysics = NULL;
	PxDefaultCpuDispatcher*	gDispatcher = NULL;
	PxScene*				gScene = NULL;
	PxMaterial*				gMaterial = NULL;
	PxReal stackZ = 10.0f;
};

