#include "Camera.h"
#include <iostream>

using namespace babyOnWheels;
using namespace scene;

Camera::Camera(glm::mat4& matrix, float _fov)
	:SceneObject(matrix),
	fov(_fov)
{
	moveVector = glm::vec3(0);
	rotationVector = glm::vec3(0);
	
	
}

void Camera::update(float time_delta) {

	//Follow the given taregt
	if (target != nullptr) {
		glm::vec3 moveVector = glm::vec3(0);
		//Follow Gamecamera - z - axis
		moveVector.z = -1 * (target->modelMatrix[3].z + followDistance + modelMatrix[3].z) * followSpeed;
		//Follow Gamecamera - x - axis
		moveVector.x = -1 * (target->modelMatrix[3].x + modelMatrix[3].x) * followSpeed;
		//Follow Gamecamera - x - axis
		moveVector.y = -1 * (target->modelMatrix[3].y - followDistance/2 +modelMatrix[3].y) * followSpeed;
		//move Camera
		move(moveVector);


	}

	//Move the Camera

	modelMatrix = glm::translate(glm::mat4(1), moveVector * time_delta) * modelMatrix;

	modelMatrix = glm::rotate(glm::mat4(1), rotationVector.y * time_delta, glm::vec3(0, 1, 0)) * modelMatrix;
	modelMatrix = glm::rotate(glm::mat4(1), rotationVector.x * time_delta, glm::vec3(1, 0, 0)) * modelMatrix;

}

void Camera::bind(GLuint handle) {
	//set up view_projection for shader
	auto proj = projectionMatrix;
	auto view = modelMatrix;
	auto view_proj = proj * view;
	auto view_proj_location = glGetUniformLocation(handle, "view_proj");
	glUniformMatrix4fv(view_proj_location, 1, GL_FALSE, glm::value_ptr(view_proj));
}

/*	Makes the camera follow the given target
	_target = the target to follow
	_followSpeed = speed of the camera
	_followDistance = distance to target
*/
void Camera::followTarget(SceneObject* _target, float _followSpeed, float _followDistance) {
	target = _target;
	followSpeed = _followSpeed;
	followDistance = _followDistance;
}

void Camera::move(glm::vec3 _dir) {
	moveVector = _dir;
}

void Camera::rotate(glm::vec3 _dir) {
	rotationVector = _dir;
}

void Camera::draw(GLuint handle) {
	
}

glm::mat4 Camera::getProjMatrix() {
	return projectionMatrix;
}

void Camera::init(GLFWwindow* window) {
	glfwGetWindowSize(window, &width, &height);
	projectionMatrix = glm::perspective(fov, width / (float)height, 0.1f, 200.0f);
}

Camera::~Camera()
{
}
