#pragma once

#include <GL\glew.h>
#include <GLFW\glfw3.h>
#include <glm\glm.hpp>
#include <glm\gtc\matrix_transform.hpp>
#include <glm\gtc\type_ptr.hpp>

#include "SceneObject.h"
#include "../Shader.h"

namespace babyOnWheels {
	namespace scene {
		class Camera : public SceneObject
		{
		public:
			Camera(glm::mat4& matrix, float _fov);
			virtual ~Camera();

			virtual void update(float time_delta);
			virtual void draw(GLuint handle);
			virtual void init(GLFWwindow* window);
			virtual void move(glm::vec3 _dir);
			virtual void rotate(glm::vec3 _dir);
			virtual void bind(GLuint handle);
			virtual void followTarget(SceneObject* _target, float _followSpeed, float _followDistance);
			glm::mat4 getProjMatrix();

		private:
			// Initial position : on +Z
			glm::vec3 position = glm::vec3(0, 0, 5);
			// Initial horizontal angle : toward -Z
			float horizontalAngle = 3.14f;
			// Initial vertical angle : none
			float verticalAngle = 0.0f;
			// Initial Field of View
			float initialFoV = 45.0f;
			float speed = 3.0f; // 3 units / second
			float mouseSpeed = 0.005f;


			glm::vec3 moveVector;
			glm::vec3 rotationVector;
			glm::mat4 projectionMatrix;
			float fov;
			int width, height;

			//Following Camera
			SceneObject* target = nullptr;
			float followSpeed;
			float followDistance;
		};
	}
}
