#pragma comment(lib, "assimp.lib")

#include "ModelLoader.h"

#include <assimp\Importer.hpp>
#include <assimp\scene.h>
#include <assimp\postprocess.h>

#include <iostream>


using namespace Assimp;


ModelLoader::ModelLoader()
{
}

btBvhTriangleMeshShape* ModelLoader::loadModel(const std::string& pathFile, GLuint vertexBuffer, GLuint normalBuffer, GLuint uvBuffer)
{
	/*

	// Create an instance of the Importer class
	Importer importer;

	// check if file exists
	std::ifstream fin(pathFile.c_str());
	if (!fin.fail()) {
		fin.close();
	}
	else{
		std::cout << "Error: could not find file in path" << std::endl;
		return 0;
	}
	
	//read the scene from the filePath with triangluation and pre transformed vertices
	const aiScene* scene = importer.ReadFile(pathFile,
		aiProcess_Triangulate |
		aiProcess_PreTransformVertices);

	// If the import failed, report it
	if (!scene)
	{
		std::cout << "Error: no scene loaded in importFile" << std::endl;
		std::cout << importer.GetErrorString() << std::endl;
		return 0;
	}

	// Now we can access the file's contents. 
	std::cout << "scene import successfull" << std::endl;

	//count number of Faces in the Scene
	unsigned int facesTotal = 0;
	for (int i = 0; i < scene->mNumMeshes; i++){
		facesTotal += scene->mMeshes[i]->mNumFaces;
	}

	//updates faces drawn so far in the loop
	//used as offset for arrays
	int facesSoFar = 0;

	GLfloat *vertexArray;
	GLfloat *normalArray;
	GLfloat *uvArray;

	// create arrays for extracting data out of the scene
	vertexArray = new GLfloat[facesTotal * 3 * 3];
	normalArray = new GLfloat[facesTotal * 3 * 3];
	uvArray = new GLfloat[facesTotal * 3 * 2];

	//do for all meshes of the scene
	for (int i = 0; i < scene->mNumMeshes; i++)
	{

		//meshes for accessing the data
		const aiMesh *mesh = scene->mMeshes[i];

		//do for all Vertices out of the mesh
		for (unsigned int j = 0; j < mesh->mNumVertices; j++){
			if (mesh->HasPositions())
			{
				//extract the vector for Vertex i and read out x,y,z
				aiVector3D vert = mesh->mVertices[j];
				vertexArray[j * 3 + (facesSoFar * 3 * 3)] = vert.x;
				vertexArray[(j * 3) + 1 + (facesSoFar * 3 * 3)] = vert.y;
				vertexArray[(j * 3) + 2 + (facesSoFar * 3 * 3)] = vert.z;

			}
			else
			{
				std::cout << "Mesh has no Positions" << std::endl;
			}
			if (mesh->HasNormals())
			{
				//extract the vector for Vertex i and read out x,y,z
				aiVector3D normals = mesh->mNormals[j];
				normalArray[j * 3 + (facesSoFar * 3 * 3)] = normals.x;
				normalArray[j * 3 + 1 + (facesSoFar * 3 * 3)] = normals.y;
				normalArray[j * 3 + 2 + (facesSoFar * 3 * 3)] = normals.z;
			}
			else
			{
				std::cout << "Mesh has no Normals" << std::endl;
			}
			if (mesh->HasTextureCoords(0))
			{
				//extract the vector for Vertex i and read out x,y (u,v coords)
				aiVector3D textureCoords = mesh->mTextureCoords[0][j];
				uvArray[j * 2 + (facesSoFar * 3 * 2)] = textureCoords.x;
				uvArray[j * 2 + 1 + (facesSoFar * 3 * 2)] = textureCoords.y;
			}

		}

		//bind buffers and add data to them
		glBindBuffer(GL_ARRAY_BUFFER, vertexBuffer);
		glBufferData(GL_ARRAY_BUFFER, facesTotal * 3 * 3 * sizeof(GLfloat), vertexArray, GL_STATIC_DRAW);

		glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
		glBufferData(GL_ARRAY_BUFFER, facesTotal * 3 * 3 * sizeof(GLfloat), normalArray, GL_STATIC_DRAW);

		glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
		glBufferData(GL_ARRAY_BUFFER, facesTotal * 3 * 2 * sizeof(GLfloat), uvArray, GL_STATIC_DRAW);

		//updates faces drawn so far in the loop
		//used as offset for arrays
		facesSoFar += scene->mMeshes[i]->mNumFaces;

	}

	btTriangleMesh* tMesh = new btTriangleMesh();
	for (int i = 0; i < facesTotal * 3 * 3; i++)
	{
		if (i % 9 == 0)
		{
			//add triangles out of x,y,z coordinate for 3 vertices and add them to the mesh
			tMesh->addTriangle(btVector3(vertexArray[i], vertexArray[i + 1], vertexArray[i + 2]), btVector3(vertexArray[i + 3], vertexArray[i + 4], vertexArray[i + 5]), btVector3(vertexArray[i + 6], vertexArray[i + 7], vertexArray[i + 8]));
		}
	}

	return new btBvhTriangleMeshShape(tMesh, true);
	*/

return nullptr;

}

ModelLoader::~ModelLoader()
{
}


bool ModelLoader::loadAssImp(
	const char * path,
	std::vector<unsigned short> & indices,
	std::vector<glm::vec3> & vertices,
	std::vector<glm::vec2> & uvs,
	std::vector<glm::vec3> & normals
	){

	Assimp::Importer importer;

	const aiScene* scene = importer.ReadFile(path, 0);
	if (!scene) {
		fprintf(stderr, importer.GetErrorString());
		getchar();
		return false;
	}

	//TODO: ber alle objekte itterieren, zur zeit wird nur das erste objekt aus der scene geladen (momentan wird durch schleife letztes Mesh verwendet)
	for (size_t j = 0; j < scene->mNumMeshes; j++)
	{
		const aiMesh* mesh = scene->mMeshes[j];

		// vertices laden
		vertices.reserve(mesh->mNumVertices);
		for (unsigned int i = 0; i < mesh->mNumVertices; i++){
			aiVector3D pos = mesh->mVertices[i];
			vertices.push_back(glm::vec3(pos.x, pos.y, pos.z));
		}

		// uv daten laden
		uvs.reserve(mesh->mNumVertices);
		for (unsigned int i = 0; i < mesh->mNumVertices; i++){
			aiVector3D UVW = mesh->mTextureCoords[0][i]; // Assume only 1 set of UV coords; AssImp supports 8 UV sets.
			uvs.push_back(glm::vec2(UVW.x, UVW.y));
		}

		// normalen laden
		normals.reserve(mesh->mNumVertices);
		for (unsigned int i = 0; i < mesh->mNumVertices; i++){
			aiVector3D n = mesh->mNormals[i];
			normals.push_back(glm::vec3(n.x, n.y, n.z));
		}

		// faces laden
		indices.reserve(3 * mesh->mNumFaces);
		for (unsigned int i = 0; i < mesh->mNumFaces; i++){
			indices.push_back(mesh->mFaces[i].mIndices[0]);
			indices.push_back(mesh->mFaces[i].mIndices[1]);
			indices.push_back(mesh->mFaces[i].mIndices[2]);
		}

	}
	return true;
}