#include "Shader.h"
#include <iostream>
#include <fstream>

using namespace babyOnWheels;

Shader::Shader(const std::string& vertexShader, const std::string& fragmentShader)
	:programHandle(0), vertexHandle(0), fragmentHandle(0)
{
	programHandle =	glCreateProgram();

	if(programHandle == 0) {
		std::cout << "Can't create handle!" << std:: endl;
		exit(EXIT_FAILURE);
	}

	loadShader(vertexShader, GL_VERTEX_SHADER, vertexHandle);
	loadShader(fragmentShader, GL_FRAGMENT_SHADER, fragmentHandle);

	link();
}


Shader::~Shader()
{
	glDeleteProgram(programHandle);
	glDeleteShader(vertexHandle);
	glDeleteShader(fragmentHandle);
}

void Shader::useShader() const {
	glUseProgram(programHandle);
}

void Shader::loadShader(const std::string& shader, GLenum shaderType, GLuint& handle){
	std::ifstream shaderFile(shader);

	if(!shaderFile.good()) {
		std::cout << "Can't load File!" << shader << std:: endl;
		system("pause");
		exit(EXIT_FAILURE);
	}

	std::string code = std::string(
		std::istreambuf_iterator<char>(shaderFile), 
		std::istreambuf_iterator<char>());
	shaderFile.close();

	handle = glCreateShader(shaderType);

	if(handle == 0) {
		std::cout << "Can't create Shader!" << std:: endl;
		system("pause");
		exit(EXIT_FAILURE);
	}

	const char * codePtr = code.c_str();
	glShaderSource(handle, 1, &codePtr, nullptr);
	glCompileShader(handle);

	GLint succeeded;
	glGetShaderiv(handle, GL_COMPILE_STATUS, &succeeded);

	if(!succeeded) {
		GLint logSize;
		glGetShaderiv(handle, GL_INFO_LOG_LENGTH, &logSize); 

		GLchar* msg = new GLchar[logSize];
		glGetShaderInfoLog(handle, logSize, nullptr, msg);

		std::cout << msg <<std::endl;
		delete[] msg;
		system("pause");
		exit(EXIT_FAILURE);
	}
}

void Shader::link() {
	glAttachShader(programHandle, vertexHandle);
	glAttachShader(programHandle, fragmentHandle);

	glLinkProgram(programHandle);

	GLint succeeded;
	glGetProgramiv(programHandle, GL_COMPILE_STATUS, &succeeded);

	if(!succeeded) {
		GLint logSize;
		glGetProgramiv(programHandle, GL_INFO_LOG_LENGTH, &logSize); 

		GLchar* msg = new GLchar[logSize];
		glGetProgramInfoLog(programHandle, logSize, nullptr, msg);

		std::cout << msg <<std::endl;
		delete[] msg;
		exit(EXIT_FAILURE);
	}


}