#version 330 core

layout(points) in;
layout(triangle_strip, max_vertices = 4) out;

uniform mat3 Modelview;
uniform mat4 ModelviewProjection;
uniform float PointSize;

in vec3 vPosition[1];
in vec3 vVelocity[1];
in float vAlpha[1];
in float vBirthTime[1];
out float gAlpha;
out vec2 gTexCoord;

const float Epsilon = 0.0000001;

void main()
{
    if (vBirthTime[0] == 0.0)
        return;

    vec3 u = Modelview * vVelocity[0];
    float w = PointSize;
    float h = w * 5.0;

    // Determine 't', which represents Z-aligned magnitude.
    // By default, t = 0.0.
    // If velocity aligns with Z, increase t towards 1.0.
    // If total speed is negligible, increase t towards 1.0.
    float t = 0.0;
    float nz = abs(normalize(u).z);
    if (nz > 1.0 - Epsilon)
        t = (nz - (1.0 - Epsilon)) / Epsilon;
    else if (dot(u,u) < Epsilon)
        t = (Epsilon - dot(u,u)) / Epsilon;

    // Compute screen-space velocity:
    u.z = 0.0;
    u = normalize(u);

    // Lerp the orientation vector if screen-space velocity is negligible:
    u = normalize(mix(u, vec3(1,0,0), t));
    h = mix(h, w, t);

    // Compute the change-of-basis matrix for the billboard:
    vec3 v = vec3(-u.y, u.x, 0);
    vec3 a = u * Modelview;
    vec3 b = v * Modelview;
    vec3 c = cross(a, b);
    mat3 basis = mat3(a, b, c);

    // Compute the four offset vectors:
    vec3 N = basis * vec3(0,w,0);
    vec3 S = basis * vec3(0,-w,0);
    vec3 E = basis * vec3(-h,0,0);
    vec3 W = basis * vec3(h,0,0);
    
    // Emit the quad:
    vec3 p = vPosition[0];
    gAlpha = vAlpha[0];
    gTexCoord = vec2(0,0); gl_Position = ModelviewProjection * vec4(p+N+W,1); EmitVertex();
    gTexCoord = vec2(1,0); gl_Position = ModelviewProjection * vec4(p+N+E,1); EmitVertex();
    gTexCoord = vec2(0,1); gl_Position = ModelviewProjection * vec4(p+S+W,1); EmitVertex();
    gTexCoord = vec2(1,1); gl_Position = ModelviewProjection * vec4(p+S+E,1); EmitVertex();
    EndPrimitive();
}
