#version 330 core

// Interpolated values from the vertex shaders
in vec2 UV_coord;
in vec3 Position_worldspace;
in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;
in vec4 ShadowCoord;

// Ouput data
layout (location = 0) out vec4 color;
layout (location = 1) out vec3 BrightColor;  

struct directionalLight 
{ 
   vec3 color; 
   vec3 dir; 
   vec3 ambientIntenstity; 
};

struct Material 
{ 
   float specPower; 
   float specShininess;
   vec3 specColor; 
   vec3 baseColor;
};

uniform directionalLight sunLight;
uniform Material mat;
uniform sampler2D myTextureSampler;
uniform mat4 MV;
uniform vec3 LightPosition_worldspace;
uniform sampler2DShadow shadowMap;

vec2 poissonDisk[16] = vec2[]( 
   vec2( -0.94201624, -0.39906216 ), 
   vec2( 0.94558609, -0.76890725 ), 
   vec2( -0.094184101, -0.92938870 ), 
   vec2( 0.34495938, 0.29387760 ), 
   vec2( -0.91588581, 0.45771432 ), 
   vec2( -0.81544232, -0.87912464 ), 
   vec2( -0.38277543, 0.27676845 ), 
   vec2( 0.97484398, 0.75648379 ), 
   vec2( 0.44323325, -0.97511554 ), 
   vec2( 0.53742981, -0.47373420 ), 
   vec2( -0.26496911, -0.41893023 ), 
   vec2( 0.79197514, 0.19090188 ), 
   vec2( -0.24188840, 0.99706507 ), 
   vec2( -0.81409955, 0.91437590 ), 
   vec2( 0.19984126, 0.78641367 ), 
   vec2( 0.14383161, -0.14100790 ) 
);

void main(){	
	vec3 diffColor = texture( myTextureSampler, UV_coord ).rgb  * mat.baseColor;
	vec3 ambientColor = sunLight.ambientIntenstity * diffColor;
	vec3 specColor = mat.specColor;

	// Normal of the computed fragment, in camera space
	vec3 n = normalize( Normal_cameraspace );
	vec3 l = normalize( LightDirection_cameraspace );
	float cosTheta = clamp( dot( n,l ), 0,1 );
	
	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_cameraspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	
	float visibility=1.0;
	float bias = 0.0005;

	// Sampling
	for (int i=0;i<10;i++){
		int index = i;
		visibility -= 0.1*(1.0-texture( shadowMap, vec3(ShadowCoord.xy + poissonDisk[index]/700.0,  (ShadowCoord.z-bias)/ShadowCoord.w) ));
	}
	color = 
		vec4(ambientColor +
		visibility * diffColor * sunLight.color * cosTheta+
		visibility * mat.specPower * specColor * sunLight.color * pow(cosAlpha,mat.specShininess),0.7);


	float brightness = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
    if(brightness >= 0.8)
        BrightColor = color.rgb;
}