#include "Texture.h"
#include <FreeImage.h>

using namespace babyOnWheels;

Texture::Texture(const std::string path)
{
	const char * path_ref = path.c_str();
	FIBITMAP* bitmap = FreeImage_Load(
		FreeImage_GetFileType(path_ref, 0),
		path_ref);

	glGenTextures(1, &handle);
	glBindTexture(GL_TEXTURE_2D, handle);

	FIBITMAP *pImage = FreeImage_ConvertTo32Bits(bitmap);
	int nWidth = FreeImage_GetWidth(pImage);
	int nHeight = FreeImage_GetHeight(pImage);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, nWidth, nHeight,
		0, GL_BGRA, GL_UNSIGNED_BYTE, (void*)FreeImage_GetBits(pImage));

	glGenerateMipmap(GL_TEXTURE_2D);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	FreeImage_Unload(bitmap);
}


Texture::~Texture(void)
{
	glDeleteTextures(1, &handle);
}

GLuint Texture::getHandle() {
	return handle;
}

void Texture::bind(int unit) {
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, handle);
}

