#include <iostream>
#include <vector>
#include <GL\glew.h>
#include <GLFW\glfw3.h>
#include <glm\glm.hpp>
#include <glm\gtc\matrix_transform.hpp>
#include <glm\gtc\type_ptr.hpp>
#include "Shader.h"
#include "Scene\Cube.h"
#include "Scene\generateViewChangedObject.h"
#include "Texture.h"
#include "Scene\Camera.h"
#include "Scene\Light.h"
#include "Entities\Player.h"
#include <sstream>
#include "Scene/ModelLoader.h"
#include "FrameBufferObject.h"
#include "Scene/SceneModel.h"
#include "PhysxManager.h"
#include "Scene/Finish.h"
#include "ParticleSystem.h"
#include "Scene/Item.h"

#include <map>
#include <string>
#include <sstream>

using namespace std;
using namespace babyOnWheels;
using namespace babyOnWheels::scene;
using namespace babyOnWheels::entities;

void init(GLFWwindow* window);
void update();
void draw();
void cleanup();
void handleInputs(GLFWwindow* window, int key, int scancode, int action, int mods);
GLuint LoadProgram(const std::string& vs, const std::string& gs, const std::string& fs);
GLuint LoadProgram(const std::string& vs);

GLuint glErr;

#define NORMAL_VIEW 0.0f
#define WIDE_VIEW 1.0f
#define CLOSE_VIEW 2.0f


enum GameStatus
{
	PLAY,
	WIN,
	LOSE
};
GameStatus actualGameStatus = GameStatus::PLAY;

enum ShowStatus
{
	Frametime,
	ObjectCount,
	NONE
};

enum TextureSampling
{
	Nearest,
	Linear
};
TextureSampling actualSampling = TextureSampling::Linear;

enum MipMapping
{
	offMip,
	NearestMip,
	LinearMip
};
MipMapping actualMipMapping = MipMapping::LinearMip;

ShowStatus actualShowStatus = ShowStatus::NONE;

bool transparent = true;

//Window
GLFWwindow* window;
//Physic
PhysxManager* physxManager;

//Shaders
Shader* shader;
Shader* depthShader;
Shader* viewShader;
Shader* blurShader;
Shader* simpleShader;

//FBOs
GLuint ShadowFrameBuffer;
GLuint depthTexture;
//GLuint viewFrameBuffer;
//GLuint viewTexture;
FrameBufferObject* viewFbo;
FrameBufferObject* pingPongFbo[2];

//Shader Variable Handles
GLuint simpleShaderTexID;
GLuint blurTextureID;
GLuint blurHorizontalID;
GLuint viewTexId;
GLuint bloomTexId;
GLuint currentFpsID;
GLuint viewDepthTexId;
GLuint invMVID;
GLuint prevMVPID;
GLuint viewRayID;

GLuint depthMatrixID;
GLuint depthModelMatrixID;

GLuint TextureID;
GLuint MatrixID;
GLuint ViewMatrixID;
GLuint ModelMatrixID;
GLuint DepthBiasID;
GLuint ShadowMapID;
GLuint lightInvDirID;

GLuint modelviewProjectionID;
GLuint timeIDparticles;
GLuint fadeRateID;
GLuint modelviewID;
GLuint colorID;
GLuint depthSamplerID;
GLuint spriteSamplerID;
GLuint pointSizeID;
GLuint inverseSizeID;

GLuint sizeID;
GLuint extentID;
GLuint timeIDadvection;
GLuint timeStepID;

ParticleSystem particleSystem;

//Sceneobjects
std::map<string, SceneModel*> sceneModels_Map;
std::map<string, SceneModel*> itemModels_Map;
SceneModel* playerModel;
Finish* finish;
SceneModel* wingHouse;
SceneModel* wingPlate;
SceneModel* wing1;
SceneModel* wing2;

//Textures
std::map<string, Texture*> textures_Map;

//Entities
Player* player;

//Lights
Light* sunLight;

//Cameras
Camera* freeCamera;
Camera* gameCamera;
Camera* activeCamera;

//ParticleEffect
GLuint ParticleBufferA;
GLuint ParticleBufferB;
const int particleCount = 1000;
Cube* particles[particleCount];
static GLuint ParticleProgram;
static GLuint AdvectProgram;

static float time = 0;
float time_delta = 0;
double mousePosX = 0, mousePosY = 0;
float MouseXvalue = 0;
float MouseYvalue = 0;

float camera_type = NORMAL_VIEW;

bool fps_on = false;
double fps_time;
double fps_lastTime = 0.0;
double fps_deltaT;
double fps_inSecond = 0.0;
int fps_frame;
int fps_frame_last;

bool viewFrustumCullingOn = false;
int counter = 0;
float** frustum;


enum AttributeSlot {
	SlotPosition,
	SlotTexCoord,
	SlotNormal,
	SlotBirthTime,
	SlotVelocity,
};

bool wireFrame = false;

float** ExtractFrustum()
{
	float** frustum = new float*[6];
	for (int i = 0; i<6; i++) frustum[i] = new float[4];
	glm::mat4 projection = activeCamera->getProjMatrix();
	glm::mat4 view = activeCamera->modelMatrix;
	float   proj[16];
	float   modl[16];
	float   clip[16];
	float   t;

	proj[0] = projection[0].x;
	proj[1] = projection[0].y;
	proj[2] = projection[0].z;
	proj[3] = projection[0].w;
	proj[4] = projection[1].x;
	proj[5] = projection[1].y;
	proj[6] = projection[1].z;
	proj[7] = projection[1].w;
	proj[8] = projection[2].x;
	proj[9] = projection[2].y;
	proj[10] = projection[2].z;
	proj[11] = projection[2].w;
	proj[12] = projection[3].x;
	proj[13] = projection[3].y;
	proj[14] = projection[3].z;
	proj[15] = projection[3].w;

	modl[0] = view[0].x;
	modl[1] = view[0].y;
	modl[2] = view[0].z;
	modl[3] = view[0].w;
	modl[4] = view[1].x;
	modl[5] = view[1].y;
	modl[6] = view[1].z;
	modl[7] = view[1].w;
	modl[8] = view[2].x;
	modl[9] = view[2].y;
	modl[10] = view[2].z;
	modl[11] = view[2].w;
	modl[12] = view[3].x;
	modl[13] = view[3].y;
	modl[14] = view[3].z;
	modl[15] = view[3].w;



	/* Combine the two matrices (multiply projection by modelview) */
	clip[0] = modl[0] * proj[0] + modl[1] * proj[4] + modl[2] * proj[8] + modl[3] * proj[12];
	clip[1] = modl[0] * proj[1] + modl[1] * proj[5] + modl[2] * proj[9] + modl[3] * proj[13];
	clip[2] = modl[0] * proj[2] + modl[1] * proj[6] + modl[2] * proj[10] + modl[3] * proj[14];
	clip[3] = modl[0] * proj[3] + modl[1] * proj[7] + modl[2] * proj[11] + modl[3] * proj[15];

	clip[4] = modl[4] * proj[0] + modl[5] * proj[4] + modl[6] * proj[8] + modl[7] * proj[12];
	clip[5] = modl[4] * proj[1] + modl[5] * proj[5] + modl[6] * proj[9] + modl[7] * proj[13];
	clip[6] = modl[4] * proj[2] + modl[5] * proj[6] + modl[6] * proj[10] + modl[7] * proj[14];
	clip[7] = modl[4] * proj[3] + modl[5] * proj[7] + modl[6] * proj[11] + modl[7] * proj[15];

	clip[8] = modl[8] * proj[0] + modl[9] * proj[4] + modl[10] * proj[8] + modl[11] * proj[12];
	clip[9] = modl[8] * proj[1] + modl[9] * proj[5] + modl[10] * proj[9] + modl[11] * proj[13];
	clip[10] = modl[8] * proj[2] + modl[9] * proj[6] + modl[10] * proj[10] + modl[11] * proj[14];
	clip[11] = modl[8] * proj[3] + modl[9] * proj[7] + modl[10] * proj[11] + modl[11] * proj[15];

	clip[12] = modl[12] * proj[0] + modl[13] * proj[4] + modl[14] * proj[8] + modl[15] * proj[12];
	clip[13] = modl[12] * proj[1] + modl[13] * proj[5] + modl[14] * proj[9] + modl[15] * proj[13];
	clip[14] = modl[12] * proj[2] + modl[13] * proj[6] + modl[14] * proj[10] + modl[15] * proj[14];
	clip[15] = modl[12] * proj[3] + modl[13] * proj[7] + modl[14] * proj[11] + modl[15] * proj[15];

	/* Extract the numbers for the RIGHT plane */
	frustum[0][0] = clip[3] - clip[0];
	frustum[0][1] = clip[7] - clip[4];
	frustum[0][2] = clip[11] - clip[8];
	frustum[0][3] = clip[15] - clip[12];

	/* Normalize the result */
	t = sqrt(frustum[0][0] * frustum[0][0] + frustum[0][1] * frustum[0][1] + frustum[0][2] * frustum[0][2]);
	frustum[0][0] /= t;
	frustum[0][1] /= t;
	frustum[0][2] /= t;
	frustum[0][3] /= t;

	/* Extract the numbers for the LEFT plane */
	frustum[1][0] = clip[3] + clip[0];
	frustum[1][1] = clip[7] + clip[4];
	frustum[1][2] = clip[11] + clip[8];
	frustum[1][3] = clip[15] + clip[12];

	/* Normalize the result */
	t = sqrt(frustum[1][0] * frustum[1][0] + frustum[1][1] * frustum[1][1] + frustum[1][2] * frustum[1][2]);
	frustum[1][0] /= t;
	frustum[1][1] /= t;
	frustum[1][2] /= t;
	frustum[1][3] /= t;

	/* Extract the BOTTOM plane */
	frustum[2][0] = clip[3] + clip[1];
	frustum[2][1] = clip[7] + clip[5];
	frustum[2][2] = clip[11] + clip[9];
	frustum[2][3] = clip[15] + clip[13];

	/* Normalize the result */
	t = sqrt(frustum[2][0] * frustum[2][0] + frustum[2][1] * frustum[2][1] + frustum[2][2] * frustum[2][2]);
	frustum[2][0] /= t;
	frustum[2][1] /= t;
	frustum[2][2] /= t;
	frustum[2][3] /= t;

	/* Extract the TOP plane */
	frustum[3][0] = clip[3] - clip[1];
	frustum[3][1] = clip[7] - clip[5];
	frustum[3][2] = clip[11] - clip[9];
	frustum[3][3] = clip[15] - clip[13];

	/* Normalize the result */
	t = sqrt(frustum[3][0] * frustum[3][0] + frustum[3][1] * frustum[3][1] + frustum[3][2] * frustum[3][2]);
	frustum[3][0] /= t;
	frustum[3][1] /= t;
	frustum[3][2] /= t;
	frustum[3][3] /= t;

	/* Extract the FAR plane */
	frustum[4][0] = clip[3] - clip[2];
	frustum[4][1] = clip[7] - clip[6];
	frustum[4][2] = clip[11] - clip[10];
	frustum[4][3] = clip[15] - clip[14];

	/* Normalize the result */
	t = sqrt(frustum[4][0] * frustum[4][0] + frustum[4][1] * frustum[4][1] + frustum[4][2] * frustum[4][2]);
	frustum[4][0] /= t;
	frustum[4][1] /= t;
	frustum[4][2] /= t;
	frustum[4][3] /= t;

	/* Extract the NEAR plane */
	frustum[5][0] = clip[3] + clip[2];
	frustum[5][1] = clip[7] + clip[6];
	frustum[5][2] = clip[11] + clip[10];
	frustum[5][3] = clip[15] + clip[14];

	/* Normalize the result */
	t = sqrt(frustum[5][0] * frustum[5][0] + frustum[5][1] * frustum[5][1] + frustum[5][2] * frustum[5][2]);
	frustum[5][0] /= t;
	frustum[5][1] /= t;
	frustum[5][2] /= t;
	frustum[5][3] /= t;
	return frustum;
}

bool SphereInFrustum(float** frustum, glm::vec4 sphere)
{
	for (int i = 0; i<6; i++)
	if (frustum[i][0] * sphere.x + frustum[i][1] * sphere.y + frustum[i][2] * sphere.z + frustum[i][3] <= -sphere.w)
		return false;
	return true;
}

void main(int argc, char** argv)
{
	auto width = 800;
	auto height = 600;
	auto fullscreen = false;

	if (argc >= 3)
	{
		std::cout << "You are executing '" << argv[0] << "'" << std::endl;

		if ((std::stringstream(argv[1]) >> width).fail())
		{
			std::cerr << "ERROR: Could not parse first command-line-argument as integer." << std::endl;
			system("PAUSE");
			exit(EXIT_FAILURE);
		}
		if ((std::stringstream(argv[2]) >> height).fail())
		{
			std::cerr << "ERROR: Could not parse second command-line-argument as integer." << std::endl;
			system("PAUSE");
			exit(EXIT_FAILURE);
		}

		if (argc >= 4)
		{
			if ((std::stringstream(argv[3]) >> fullscreen).fail())
			{
				std::cerr << "ERROR: Could not parse third command-line-argument as integer." << std::endl;
				system("PAUSE");
				exit(EXIT_FAILURE);
			}
		}
	}


	//Init GLFW and create Window
	if (glfwInit() == false) {
		std::cout << "Could not init GFLW!" << std::endl;
		system("pause");
	}


	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
	glfwWindowHint(GLFW_OPENGL_DEBUG_CONTEXT, GL_TRUE);
	glfwWindowHint(GLFW_SAMPLES, 4);

	GLFWmonitor* monitor = nullptr;

	// starts as fullscreen if 1 is set in degubbing properties (0 is not fullscreen)
	if (fullscreen)
	{
		monitor = glfwGetPrimaryMonitor();

		int refresh_rate = 60;
		glfwWindowHint(GLFW_REFRESH_RATE, refresh_rate);
	}
	window = glfwCreateWindow(width, height, "Baby On Wheels", monitor, nullptr);

	if (!window) {
		std::cout << "Can't create window!" << std::endl;
		glfwTerminate();
		system("pause");
		exit(EXIT_FAILURE);
	}
	glfwMakeContextCurrent(window);
	glfwGetFramebufferSize(window, &width, &height);

	// Initialize GLEW
	glewExperimental = true; // Needed for core profile
	if (glewInit() != GLEW_OK) {
		fprintf(stderr, "Failed to initialize GLEW\n");
		getchar();
		glfwTerminate();
		exit(EXIT_FAILURE);
	}

	init(window);
	// Create Framebuffer	
	glGenFramebuffers(1, &ShadowFrameBuffer);
	glBindFramebuffer(GL_FRAMEBUFFER, ShadowFrameBuffer);

	
	glGenTextures(1, &depthTexture);
	glBindTexture(GL_TEXTURE_2D, depthTexture);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT16, 1024, 1024, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC, GL_LEQUAL);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_COMPARE_R_TO_TEXTURE);

	glFramebufferTexture(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, depthTexture, 0);
	glDrawBuffer(GL_NONE);

	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
		exit(EXIT_FAILURE);

	double time = glfwGetTime();

	while (glfwWindowShouldClose(window) == false) {
		double time_new = glfwGetTime();
		time_delta = (float)(time_new - time);

		//--------------------------------------------------------------
		//OUTPUT  every second
		if (((int)time) != ((int)time_new)) {
			std::cout << player->points << std::endl;
			if (actualShowStatus == ShowStatus::Frametime) {
				std::cout << "frametime: " << time_delta * 1000 << "ms"
					<< " *~ " << 1.0f / time_delta << "fps" << std::endl;
			}
			else if (actualShowStatus == ShowStatus::ObjectCount) {
				std::cout << "Drawn Models: " << counter << std::endl;
				std::cout << "Carriage position: " << playerModel->getSphere().x << " " << playerModel->getSphere().y << " " << playerModel->getSphere().z << " " << playerModel->getSphere().w << std::endl;

			}
		}
		//--------------------------------------------------------------

		time = time_new;

		
		glfwGetCursorPos(window, &mousePosX, &mousePosY);
		update();
		glUniform1f(currentFpsID, 1.0f / time_delta);
		draw();
		glfwSetCursorPos(window, width / 2, height / 2);
		
		//Mouse Movement
		MouseYvalue -= (float(height / 2 - mousePosY)) * time_delta;
		MouseXvalue -= (float(width / 2 - mousePosX)) * time_delta;

		//----------------------------------------------------------------------------------------------
		// Swap buffers
		glfwSwapBuffers(window);
		glfwPollEvents();
		if (glGetError() != GL_NO_ERROR) {
			std::cout << "OpenGl Error" << std::endl;
		}
		if (glfwGetKey(window, GLFW_KEY_ESCAPE)) {
			glfwSetWindowShouldClose(window, true);
		}


		/*glm::mat4 test = finish->modelMatrix;// player->getModel()->modelMatrix;// glm::mat4(1);
		//test = glm::scale(test, glm::vec3(2, 3, 4));
		for (int i = 0; i < 4; i++) {
			for (int j = 0; j < 4; j++) {
				std::cout << test[i][j]<< " | ";
			}
			std::cout << std::endl;
		}
		std::cout << std::endl;*/
	}

	cleanup();
	glfwTerminate();
	exit(EXIT_SUCCESS);
}



glm::vec3 cameraposition = glm::vec3(3,-5,0);
glm::vec3 cameraright = glm::vec3(1);
glm::vec3 cameradirection = glm::vec3(0,-1,0);
glm::vec3 cameraup = glm::vec3(1);
static void cursor_pos_callback(GLFWwindow* window, double xpos, double ypos)
{
	
	int width, height;
	glfwGetWindowSize(window, &width, &height);
	
}

void init(GLFWwindow* window) {

	glfwSetWindowTitle(window, "Baby On Wheels");
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glEnable(GL_CULL_FACE);

	GLuint VertexArrayID;
	glGenVertexArrays(1, &VertexArrayID);
	glBindVertexArray(VertexArrayID);
	
	//set Listeners
	glfwSetKeyCallback(window, handleInputs);
	glfwSetCursorPosCallback(window, cursor_pos_callback);

	//Physx
	physxManager = new PhysxManager();

	//Textures
	Texture* loadTexture = new Texture("../Baby_On_Wheels/Textures/tex.png");
	textures_Map.insert(std::pair<string, Texture*>("test", loadTexture));
	Texture* oilTexture = new Texture("../External/Objects/oildrum_col.jpg");
	textures_Map.insert(std::pair<string, Texture*>("oilbarrel", oilTexture));
	Texture* groundTexture = new Texture("../Baby_On_Wheels/Textures/ground08.jpg");
	textures_Map.insert(std::pair<string, Texture*>("ground", groundTexture));
	Texture* wallTexture = new Texture("../Baby_On_Wheels/Textures/wall__09.jpg");
	textures_Map.insert(std::pair<string, Texture*>("wall", wallTexture));
	Texture* groundTexture2 = new Texture("../Baby_On_Wheels/Textures/floor_06r.jpg");
	textures_Map.insert(std::pair<string, Texture*>("ground2", groundTexture2));
	Texture* treeTexture = new Texture("../Baby_On_Wheels/Textures/treeSimpleTex.png");
	textures_Map.insert(std::pair<string, Texture*>("tree1", treeTexture));
	Texture* winTexture = new Texture("../Baby_On_Wheels/Textures/winTexture.png");
	textures_Map.insert(std::pair<string, Texture*>("winTexture", winTexture));
	Texture* loseTexture = new Texture("../Baby_On_Wheels/Textures/loseTexture.png");
	textures_Map.insert(std::pair<string, Texture*>("loseTexture", loseTexture));
	Texture* finishTexture = new Texture("../Baby_On_Wheels/Textures/finish.png");
	textures_Map.insert(std::pair<string, Texture*>("finishTexture", finishTexture));
	Texture* milkTexture = new Texture("../Baby_On_Wheels/Textures/milk.png");
	textures_Map.insert(std::pair<string, Texture*>("milk", milkTexture));
	Texture* wagenTexture = new Texture("../Baby_On_Wheels/Textures/wagen.png");
	textures_Map.insert(std::pair<string, Texture*>("wagenTexture", wagenTexture));
	Texture* woodTexture = new Texture("../Baby_On_Wheels/Textures/wood1.png");
	textures_Map.insert(std::pair<string, Texture*>("woodTexture", woodTexture));
	Texture* wingTexture = new Texture("../Baby_On_Wheels/Textures/wing.png");
	textures_Map.insert(std::pair<string, Texture*>("wing", wingTexture));
	Texture* wingHouseTex = new Texture("../Baby_On_Wheels/Textures/wingHouse.png");
	textures_Map.insert(std::pair<string, Texture*>("wingHouse", wingHouseTex));

	//Shader
	// load all shaders
	shader = new Shader("../Baby_On_Wheels/Shader/shadow.vert", "../Baby_On_Wheels/Shader/shadow.frag");
	depthShader = new Shader("../Baby_On_Wheels/Shader/depth.vert", "../Baby_On_Wheels/Shader/depth.frag");
	viewShader = new Shader("../Baby_On_Wheels/Shader/passthrough.vert", "../Baby_On_Wheels/Shader/motionBlur.frag");
	blurShader = new Shader("../Baby_On_Wheels/Shader/passthrough.vert", "../Baby_On_Wheels/Shader/blur.frag");
	simpleShader = new Shader("../Baby_On_Wheels/Shader/passthrough.vert", "../Baby_On_Wheels/Shader/simple.frag");

	//FBOs
	int width, height;
	glfwGetWindowSize(window, &width, &height);
	viewFbo = new FrameBufferObject(0, width, height);
	pingPongFbo[0] = new FrameBufferObject(3, width, height);
	pingPongFbo[1] = new FrameBufferObject(4, width, height);

	//Lights
	sunLight = new Light(glm::vec3(1.0f, 1.0f, 0.8f), glm::vec3(-4, -15, -4), glm::vec3(0.2f,0.2f,0.25f));

	//Sceneobjects
	playerModel = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(0, -1.5f, 1)));
	playerModel->loadSceneModel("../External/Objects/Baby_Carriage2.dae", textures_Map.find("wagenTexture")->second->getHandle());
	sceneModels_Map.insert(std::pair<string, SceneModel*>("player", playerModel));
	playerModel->attachRigidBody(physxManager->createRigidbody(playerModel->modelMatrix, 40, glm::vec3(1.0f, 1.1f, 1.0f)));

	//-------------------------------------------------------------------------------------------------------------------
	//-------------------------------------------------------------------------------------------------------------------
	//Streetpart - alle teile sind unterobjekte von street
	//EBENE1
	int number = 4;
	physxManager->createStaticCollider(glm::translate(glm::mat4(1), glm::vec3(0, 0, -20-10*number)), glm::vec3(10, 0.0f, 20 * number));
	physxManager->createStaticCollider(glm::translate(glm::mat4(1), glm::vec3(-10, 0, -20 - 10 * number)), glm::vec3(0, 3, 20 * number));
	physxManager->createStaticCollider(glm::translate(glm::mat4(1), glm::vec3(10, 0, -20 - 10 * number)), glm::vec3(0, 3, 20 * number));
	for (int i = 0; i < number; i++) {
		SceneModel* street = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(0, 0, 0 - i * 40)));
		street->loadSceneModel("../Baby_On_Wheels/Objects/streetPart1.dae", textures_Map.find("ground")->second->getHandle());
		sceneModels_Map.insert(std::pair<string, SceneModel*>("street" + to_string(i), street));

		SceneModel* fence = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(10, 0, 0 - i * 40)));
		fence->loadSceneModel("../Baby_On_Wheels/Objects/fence.dae", textures_Map.find("woodTexture")->second->getHandle());
		sceneModels_Map.insert(std::pair<string, SceneModel*>("fence11_" + to_string(i), fence));

		SceneModel* fence2 = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(-10, 0, 0 - i * 40)));
		fence2->loadSceneModel("../Baby_On_Wheels/Objects/fence.dae", textures_Map.find("woodTexture")->second->getHandle());
		sceneModels_Map.insert(std::pair<string, SceneModel*>("fence12_" + to_string(i), fence2));

		//ITEMs
		for (int i = 0; i < 10; i++) {
			Item* milk = new Item(glm::translate(glm::mat4(1), glm::vec3(-10 + rand() % 20, -1, -rand() % 200)));
			milk->modelMatrix = glm::rotate(milk->modelMatrix, glm::radians(45.0f), glm::vec3(0, 0, 1));
			milk->trigger = glm::scale(glm::translate(milk->modelMatrix, glm::vec3(0, 0, 0)), glm::vec3(1.5f, 1.5f, 1.5f));
			milk->loadSceneModel("../Baby_On_Wheels/Objects/flaschl.dae", textures_Map.find("milk")->second->getHandle());
			itemModels_Map.insert(std::pair<string, SceneModel*>("milk" + to_string(i), milk));
		}
	}
	//EBENE2
	int posy = -40 * number;
	number = 4;
	physxManager->createStaticCollider(glm::translate(glm::mat4(1), glm::vec3(5, 3, posy -20 - 10 * number)), glm::vec3(10, 0.0f, 20 * number));
	physxManager->createStaticCollider(glm::translate(glm::mat4(1), glm::vec3(-5, 3, posy  - 20 - 10 * number)), glm::vec3(0, 3, 20 * number));
	physxManager->createStaticCollider(glm::translate(glm::mat4(1), glm::vec3(15, 3, posy  - 20 - 10 * number)), glm::vec3(0, 3, 20 * number));
	for (int i = 0; i < number; i++) {
		SceneModel* street = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(5, 3, posy - i * 40)));
		street->loadSceneModel("../Baby_On_Wheels/Objects/streetPart1.dae", textures_Map.find("ground")->second->getHandle());
		sceneModels_Map.insert(std::pair<string, SceneModel*>("street2" + to_string(i), street));

		SceneModel* fence = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(15, 3, posy - i * 40)));
		fence->loadSceneModel("../Baby_On_Wheels/Objects/fence.dae", textures_Map.find("woodTexture")->second->getHandle());
		sceneModels_Map.insert(std::pair<string, SceneModel*>("fence21_" + to_string(i), fence));

		SceneModel* fence2 = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(-5, 3, posy - i * 40)));
		fence2->loadSceneModel("../Baby_On_Wheels/Objects/fence.dae", textures_Map.find("woodTexture")->second->getHandle());
		sceneModels_Map.insert(std::pair<string, SceneModel*>("fence22_" + to_string(i), fence2));

		//ITEMs
		for (int i = 0; i < 10; i++) {
			Item* milk = new Item(glm::translate(glm::mat4(1), glm::vec3(-15 + rand() % 20, 3-1, posy -rand() % 200)));
			milk->modelMatrix = glm::rotate(milk->modelMatrix, glm::radians(45.0f), glm::vec3(0, 0, 1));
			milk->trigger = glm::scale(glm::translate(milk->modelMatrix, glm::vec3(0, 0, 0)), glm::vec3(1.5f, 1.5f, 1.5f));
			milk->loadSceneModel("../Baby_On_Wheels/Objects/flaschl.dae", textures_Map.find("milk")->second->getHandle());
			itemModels_Map.insert(std::pair<string, SceneModel*>("milk" + to_string(i), milk));
		}
	}
	//EBENE3
	posy -= 40 * number;
	number = 4;
	physxManager->createStaticCollider(glm::translate(glm::mat4(1), glm::vec3(0, 7, posy - 20 - 10 * number)), glm::vec3(10, 0.0f, 20 * number));
	physxManager->createStaticCollider(glm::translate(glm::mat4(1), glm::vec3(-10, 3, posy - 20 - 10 * number)), glm::vec3(0, 3, 20 * number));
	physxManager->createStaticCollider(glm::translate(glm::mat4(1), glm::vec3(10, 3, posy - 20 - 10 * number)), glm::vec3(0, 3, 20 * number));
	for (int i = 0; i < number; i++) {
		SceneModel* street = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(0, 7, posy - i * 40)));
		street->loadSceneModel("../Baby_On_Wheels/Objects/streetPart1.dae", textures_Map.find("ground")->second->getHandle());
		sceneModels_Map.insert(std::pair<string, SceneModel*>("street3" + to_string(i), street));

		SceneModel* fence = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(10, 7, posy - i * 40)));
		fence->loadSceneModel("../Baby_On_Wheels/Objects/fence.dae", textures_Map.find("woodTexture")->second->getHandle());
		sceneModels_Map.insert(std::pair<string, SceneModel*>("fence31_" + to_string(i), fence));

		SceneModel* fence2 = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(-10, 7, posy - i * 40)));
		fence2->loadSceneModel("../Baby_On_Wheels/Objects/fence.dae", textures_Map.find("woodTexture")->second->getHandle());
		sceneModels_Map.insert(std::pair<string, SceneModel*>("fence32_" + to_string(i), fence2));

		//ITEMs
		for (int i = 0; i < 10; i++) {
			Item* milk = new Item(glm::translate(glm::mat4(1), glm::vec3(-10 + rand() % 20, 7 - 1, posy - rand() % 200)));
			milk->modelMatrix = glm::rotate(milk->modelMatrix, glm::radians(45.0f), glm::vec3(0, 0, 1));
			milk->trigger = glm::scale(glm::translate(milk->modelMatrix, glm::vec3(0, 0, 0)), glm::vec3(1.5f, 1.5f, 1.5f));
			milk->loadSceneModel("../Baby_On_Wheels/Objects/flaschl.dae", textures_Map.find("milk")->second->getHandle());
			itemModels_Map.insert(std::pair<string, SceneModel*>("milk" + to_string(i), milk));
		}
	}
	//########################################################################################

	//##################################################################
	//Hierachisches Objekt
	wingHouse = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(0, 0, -12)));
	wingHouse->loadSceneModel("../Baby_On_Wheels/Objects/wingHouse.dae", textures_Map.find("wingHouse")->second->getHandle());
	sceneModels_Map.insert(std::pair<string, SceneModel*>("wingHouse", wingHouse));
	wingPlate = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(0, -10, -10)));
	wingPlate->loadSceneModel("../Baby_On_Wheels/Objects/wingPlate.dae", textures_Map.find("wing")->second->getHandle());
	wingHouse->sceneModels_Map.insert(std::pair<string, SceneModel*>("wingPlate", wingPlate));
	wing1 = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(3, -10, -10)));
	wing1->loadSceneModel("../Baby_On_Wheels/Objects/wing.dae", textures_Map.find("wing")->second->getHandle());
	wingPlate->sceneModels_Map.insert(std::pair<string, SceneModel*>("wing1", wing1));
	wing2 = new SceneModel(glm::translate(glm::mat4(1), glm::vec3(-3, -10, -10)));
	wing2->loadSceneModel("../Baby_On_Wheels/Objects/wing.dae", textures_Map.find("wing")->second->getHandle());
	wingPlate->sceneModels_Map.insert(std::pair<string, SceneModel*>("wing2", wing2));

	wingHouse->translate(glm::vec3(20, 0, -20));
	//##################################################################

	finish = new Finish(glm::translate(glm::mat4(1), glm::vec3(0, 7, -450)));
	finish->trigger = glm::scale(glm::translate(finish->modelMatrix, glm::vec3(0, 0, -1)), glm::vec3(10, 10, 1));
	finish->loadSceneModel("../Baby_On_Wheels/Objects/finish.dae", textures_Map.find("finishTexture")->second->getHandle());
	//finish->attachCollider(physxManager->createStaticCollider(finish->modelMatrix, glm::vec3(5, 5, 5)));
	sceneModels_Map.insert(std::pair<string, SceneModel*>("finish", finish));


	//Jumps
	//RIGIDBODYs
	for (int i = 0; i < 20; i++) {

		SceneModel *woodBox = new SceneModel(
			glm::translate(glm::mat4(1), glm::vec3(-20 + rand() % 40, -1, -rand() % 200)));
		woodBox->loadSceneModel("../Baby_On_Wheels/Objects/cube.dae", textures_Map.find("woodTexture")->second->getHandle());
		woodBox->attachRigidBody(physxManager->createRigidbody(woodBox->modelMatrix, 2.0f, glm::vec3(0.8f, 0.8f, 0.8f)));
		sceneModels_Map.insert(std::pair<string, SceneModel*>("woodBox1_" + to_string(i), woodBox));
	}
	
	//Entities
	player = new Player(playerModel, shader);	

	//Cameras
	gameCamera = new Camera(glm::translate(glm::mat4(1.0f), glm::vec3(0, 2.0f, -9)), 30);
	gameCamera->init(window);
	gameCamera->followTarget(player->getModel(), 20, 8); // follow distance 8
	freeCamera = new Camera(glm::mat4(1.0f), 30);
	freeCamera->init(window);
	activeCamera = gameCamera;

	// Get all the handles forthe shaders
	simpleShaderTexID = glGetUniformLocation(simpleShader->programHandle, "image");
	blurTextureID = glGetUniformLocation(viewShader->programHandle, "image");
	blurHorizontalID = glGetUniformLocation(viewShader->programHandle, "horizontal");
	invMVID = glGetUniformLocation(viewShader->programHandle, "invMV");
	prevMVPID = glGetUniformLocation(viewShader->programHandle, "prevMVP");
	viewRayID = glGetUniformLocation(viewShader->programHandle, "viewRay");
	viewTexId = glGetUniformLocation(viewShader->programHandle, "renderedTexture");
	bloomTexId = glGetUniformLocation(viewShader->programHandle, "brightTexture");
	currentFpsID = glGetUniformLocation(viewShader->programHandle, "currentFps");
	viewDepthTexId = glGetUniformLocation(viewShader->programHandle, "depthTexture");

	depthMatrixID = glGetUniformLocation(depthShader->programHandle, "depthVP");
	depthModelMatrixID = glGetUniformLocation(depthShader->programHandle, "M");
	TextureID = glGetUniformLocation(shader->programHandle, "myTextureSampler");
	MatrixID = glGetUniformLocation(shader->programHandle, "VP");
	ViewMatrixID = glGetUniformLocation(shader->programHandle, "V");
	ModelMatrixID = glGetUniformLocation(shader->programHandle, "M");
	DepthBiasID = glGetUniformLocation(shader->programHandle, "DepthBiasVP");
	ShadowMapID = glGetUniformLocation(shader->programHandle, "shadowMap");
	lightInvDirID = glGetUniformLocation(shader->programHandle, "LightInvDirection_worldspace");

	//Init particle system

	// extracting the position from the babycarriage



	glm::mat4 mat = glm::mat4(1, 5, 9, 13,
		2, 6, 10, 14,
		3, 7, 11, 15,
		4, 8, 12, 16);

	mat[0][3] = cameraposition.x;
	mat[1][3] = cameraposition.y;
	mat[2][3] = cameraposition.z;

	// cout << pos.x << pos.y << pos.z << endl;

	// particleSystem.InitParticleSystem(&(mat));
	particleSystem.InitParticleSystem(&(playerModel->modelMatrix));

	std::cout << ((playerModel->modelMatrix))[0][3] << " " << ((playerModel->modelMatrix))[1][3] << " " << ((playerModel->modelMatrix))[2][3] << std::endl;
	// std::cout << (cameraposition.x << " " << cameraposition.y << " " << cameraposition.z << std::endl;



	// Particle Effects
	// for (size_t i = 0; i < particleCount; i++)
	// {
		//particles[i] = new Cube(glm::mat4(1.0f), shader);
	// }

	// fr die variante mit geometry und fragment shader
	/*
	ParticleProgram = LoadProgram("../Baby_On_Wheels/Shader/particle.frag", "../Baby_On_Wheels/Shader/particle_geometry.vert", "../Baby_On_Wheels/Shader/particle.vert");
	modelviewProjectionID = glGetUniformLocation(ParticleProgram, "ModelviewProjection");
	timeIDparticles = glGetUniformLocation(ParticleProgram, "Time");
	fadeRateID = glGetUniformLocation(ParticleProgram, "FadeRate");
	modelviewID = glGetUniformLocation(ParticleProgram, "Modelview");
	// TODO Set Variables for Particle Program
	// glUniformMatrix4fv(depthMatrixID, 1, GL_FALSE, &depthMVP[0][0]);
	*/

	/*ParticleProgram = LoadProgram("../Baby_On_Wheels/Shader/particle.vert");
	// Bind outputs from vertex (geometry) shader
	// const char* varyings[3] = { "vPosition", "vBirthTime", "vVelocity" };
	// glTransformFeedbackVaryings(ParticleProgram, 3, varyings, GL_INTERLEAVED_ATTRIBS);
	
	modelviewProjectionID = glGetUniformLocation(ParticleProgram, "ModelviewProjection");
	timeIDparticles = glGetUniformLocation(ParticleProgram, "Time");
	fadeRateID = glGetUniformLocation(ParticleProgram, "FadeRate");
	// TODO Set Variables for Particle Program

	AdvectProgram = LoadProgram("../Baby_On_Wheels/Shader/particle_advection.vert");
	sizeID = glGetUniformLocation(AdvectProgram, "Size");
	extentID = glGetUniformLocation(AdvectProgram, "Extent");
	timeIDadvection = glGetUniformLocation(AdvectProgram, "Time");
	timeStepID = glGetUniformLocation(AdvectProgram, "TimeStep");

	// glUniform3f(sizeID, 128, 256, 128);
	// glUniform3f(extentID, 2, 4, 2);


	// Create VBO for input on even-numbered frames and output on odd-numbered frames:
	glGenBuffers(1, &ParticleBufferA);
	glBindBuffer(GL_ARRAY_BUFFER, ParticleBufferA);
	glBufferData(GL_ARRAY_BUFFER, sizeof(particles), &particles[0], GL_STREAM_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// Create VBO for output on even-numbered frames and input on odd-numbered frames:
	glGenBuffers(1, &ParticleBufferB);
	glBindBuffer(GL_ARRAY_BUFFER, ParticleBufferB);
	glBufferData(GL_ARRAY_BUFFER, sizeof(particles), 0, GL_STREAM_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);*/
}

/*
  verarbeitet keyboard eingaben
*/
void handleInputs(GLFWwindow* window, int key, int scancode, int action, int mods) {
	//glfwSetCursorPos(window, 1024 / 2, 768 / 2);
	//Toggle between cameras 1=gameCam, 2=freeCam
	if (glfwGetKey(window, GLFW_KEY_1) == GLFW_PRESS) {
		activeCamera = gameCamera;
	}
	else if (glfwGetKey(window, GLFW_KEY_2) == GLFW_PRESS) {
		activeCamera = freeCamera;
	}

	
	//Standard Player Inputs
	if (glfwGetKey(window, GLFW_KEY_UP) == GLFW_PRESS) {
		player->doAccelerate(true);
	}
	else {
		player->doAccelerate(false);
	}
	if (glfwGetKey(window, GLFW_KEY_DOWN) == GLFW_PRESS) {
		player->doBrake(true);
	}
	else {
		player->doBrake(false);
	}
	if (glfwGetKey(window, GLFW_KEY_LEFT) == GLFW_PRESS) {
		player->doSteer(-1);
	}
	if (glfwGetKey(window, GLFW_KEY_RIGHT) == GLFW_PRESS) {
		player->doSteer(1);
	}
	if (glfwGetKey(window, GLFW_KEY_LEFT) != GLFW_PRESS &&
		glfwGetKey(window, GLFW_KEY_RIGHT) != GLFW_PRESS) {
		player->doSteer(0);
	}
	if (glfwGetKey(window, GLFW_KEY_SPACE) == GLFW_PRESS) {
		player->jump();
	}

	// Help (if available)
	if (glfwGetKey(window, GLFW_KEY_F1) == GLFW_PRESS)
	{
		// TODO
		std::cout << "No help available" << std::endl;
	}

	// Frame Time on/off
	if (glfwGetKey(window, GLFW_KEY_F2) == GLFW_PRESS)
	{
		if (actualShowStatus == ShowStatus::Frametime) {
			actualShowStatus = ShowStatus::NONE;
		}
		else{
			actualShowStatus = ShowStatus::Frametime;
		}
	}

	// Wire Frame on/off
	if (glfwGetKey(window, GLFW_KEY_F3) == GLFW_PRESS)
	{
		if (wireFrame == true)
		{
			wireFrame = false;
			std::cout << "WireFrame OFF" << std::endl;
		}
		else
		{
			wireFrame = true;
			std::cout << "WireFrame ON" << std::endl;
		}

		// std::cout << "WireFrame " << wireFrame << std::endl;

	}

	// Textur-Sampling-Quality: Nearest Neighbour/Bilinear
	if (glfwGetKey(window, GLFW_KEY_F4) == GLFW_PRESS)
	{
		if (actualSampling == TextureSampling::Linear) {
			actualSampling = TextureSampling::Nearest;
			std::cout << "Textur-Sampling-Quality: Nearest Neighbour" << std::endl;
		}
		else {
			actualSampling = TextureSampling::Linear;
			std::cout << "Textur-Sampling-Quality: Bilinear" << std::endl;
		}
	}

	// Mip Mapping-Quality: Off/Nearest Neighbour/Linear
	if (glfwGetKey(window, GLFW_KEY_F5) == GLFW_PRESS)
	{
		if (actualMipMapping == MipMapping::offMip) {
			actualMipMapping = MipMapping::NearestMip;
			std::cout << "Mip Mapping: Nearest Neighbor" << std::endl;
		}
		else if (actualMipMapping == MipMapping::NearestMip){
			actualMipMapping = MipMapping::LinearMip;
			std::cout << "Mip Mapping: Linear" << std::endl;
		}
		else {
			actualMipMapping = MipMapping::offMip;
			std::cout << "Mip Mapping: off" << std::endl;
		}
	}

	// Set gameCamera to WIDE_VIEW
	if (glfwGetKey(window, GLFW_KEY_F6) == GLFW_PRESS)
	{
		if (camera_type == NORMAL_VIEW)
		{
			gameCamera->followTarget(player->getModel(), 20, 14);
			camera_type = WIDE_VIEW;
			std::cout << "Wide view" << std::endl;
		}
		else if (camera_type == CLOSE_VIEW)
		{
			gameCamera->followTarget(player->getModel(), 20, 14);
			camera_type = WIDE_VIEW;
			std::cout << "Wide view" << std::endl;
		}
		else
		{
			gameCamera->followTarget(player->getModel(), 20, 8);
			camera_type = NORMAL_VIEW;
			std::cout << "Normal view" << std::endl;
		}
	}

	// Set gameCamera to CLOSE_VIEW
	if (glfwGetKey(window, GLFW_KEY_F7) == GLFW_PRESS)
	{
		if (camera_type == NORMAL_VIEW)
		{
			gameCamera->followTarget(player->getModel(), 20, 3);
			camera_type = CLOSE_VIEW;
			std::cout << "Close view" << std::endl;
		}
		else if (camera_type == WIDE_VIEW)
		{
			gameCamera->followTarget(player->getModel(), 20, 3);
			camera_type = CLOSE_VIEW;
			std::cout << "Close view" << std::endl;
		}
		else
		{
			gameCamera->followTarget(player->getModel(), 20, 8);
			camera_type = NORMAL_VIEW;
			std::cout << "Normal view" << std::endl;
		}

	}

	// Viewfrustum-Culling on/off
	if (glfwGetKey(window, GLFW_KEY_F8) == GLFW_PRESS)
	{
		actualShowStatus = ShowStatus::ObjectCount;
		if (viewFrustumCullingOn)
		{
			viewFrustumCullingOn = false;
			actualShowStatus = ShowStatus::NONE;
			//std::cout << "Drawn Models: " << counter << std::endl;
		}
		else
		{	
			viewFrustumCullingOn = true;
			actualShowStatus = ShowStatus::ObjectCount;
			//std::cout << "Drawn Models: " << counter << std::endl;
			//std::cout << "Carriage position: " << playerModel->getSphere().x << " " << playerModel->getSphere().y << " " << playerModel->getSphere().z << " " << playerModel->getSphere().w << std::endl;
		}
	}

	// Transparency on/off
	if (glfwGetKey(window, GLFW_KEY_F9) == GLFW_PRESS)
	{
		transparent = !transparent;
		if (transparent == false)
		{
			std::cout << "Transparency off" << std::endl;
		}
		else
		{
			std::cout << "Transparency on" << std::endl;
		}
		
	}
	
	if (activeCamera == freeCamera) {
		float speed = 10;
		// Frame Time on/off
		if (glfwGetKey(window, GLFW_KEY_LEFT_SHIFT) == GLFW_PRESS)
		{
			speed *= 2;
		}
		// Move forward
		if (glfwGetKey(window, GLFW_KEY_W) == GLFW_PRESS){
			cameraposition += speed * cameradirection * time_delta;
		}
		// Move backward
		if (glfwGetKey(window, GLFW_KEY_S) == GLFW_PRESS){
			cameraposition -= speed * cameradirection * time_delta;
		}
		// Strafe right
		if (glfwGetKey(window, GLFW_KEY_D) == GLFW_PRESS){
			cameraposition += cameraright * time_delta;
		}
		// Strafe left
		if (glfwGetKey(window, GLFW_KEY_A) == GLFW_PRESS){
			cameraposition -= cameraright * time_delta;
		}
	}
}

void update() {

	//update view frustum for culling
	frustum = ExtractFrustum();

	if (actualGameStatus == GameStatus::PLAY) {
		physxManager->update(time_delta);
		for (const auto model : sceneModels_Map) {
			model.second->update(time_delta);
		}
		player->update(time_delta);
		if (finish->checkIfInside(player->getModel()->modelMatrix)) {
			actualGameStatus = GameStatus::WIN;
			return;
		}
		if (player->dead) {
			actualGameStatus = GameStatus::LOSE;
		}

		for (const auto model : itemModels_Map) {
			((Item*)model.second)->update(time_delta);
			if (((Item*)model.second)->alive && model.second->checkIfInside(player->getModel()->modelMatrix)) {
				player->points += ((Item*)model.second)->value;
				((Item*)model.second)->alive = false;
			}
		}

		wingPlate->rotate(time_delta, glm::vec3(0, 0, 1));
		wing1->rotate(2 * -time_delta, glm::vec3(0, 0, 1));
		wing2->rotate(2 * time_delta, glm::vec3(0, 0, 1));

		gameCamera->update(time_delta);


		//Freie Camera verwenden
		if (activeCamera == freeCamera) {
			cameradirection = glm::vec3(
				cos(MouseYvalue) * sin(MouseXvalue),
				sin(MouseYvalue),
				cos(MouseYvalue) * cos(MouseXvalue)
				);
			cameraright = glm::vec3(
				sin(MouseXvalue - 3.14f / 2.0f),
				0,
				cos(MouseXvalue - 3.14f / 2.0f)
				);
			cameraup = glm::cross(cameraright, cameradirection);
			freeCamera->modelMatrix = glm::lookAt(
				cameraposition,
				cameraposition + cameradirection,
				cameraup
				);
		}
	}
	else if (actualGameStatus == GameStatus::WIN)
	{
		//do Win update
	}
	else {
		//do lose update
	}

	// calculate delta time
	fps_time = glfwGetTime();
	fps_deltaT = time - fps_lastTime;
	fps_frame++;
	
	fps_inSecond += time_delta;
	// std::cout << fps_inSecond << std::endl;

	if (fps_inSecond >= 1.0)
	{
		fps_frame_last = fps_frame;
		fps_inSecond = 0;
		fps_frame = 0;
		// std::cout << fps_inSecond << std::endl;
	}
	fps_lastTime = fps_time;

}

void draw() {
	int width, height;
	glfwGetWindowSize(window, &width, &height);

	int internCounter = 0;
	
	if (actualGameStatus == GameStatus::PLAY) {

		//Berechne globale Matrizen
		glm::mat4 ProjectionMatrix = activeCamera->getProjMatrix();// getProjectionMatrix();
		glm::mat4 ViewMatrix = activeCamera->modelMatrix;
		glm::mat4 biasMatrix(
			0.5, 0.0, 0.0, 0.0,
			0.0, 0.5, 0.0, 0.0,
			0.0, 0.0, 0.5, 0.0,
			0.5, 0.5, 0.5, 1.0
			);
		glm::vec3 lightInvDir = sunLight->getRotation();
		glm::mat4 depthProjectionMatrix = glm::ortho<float>(-35, 35, -35, 35, 0, 180);
		glm::mat4 depthViewMatrix = glm::lookAt(lightInvDir + glm::vec3(glm::inverse(activeCamera->modelMatrix)[3]),
			glm::vec3(0, 0, 0) + glm::vec3(glm::inverse(activeCamera->modelMatrix)[3]), glm::vec3(0, 1, 0));

		//Clear Buffers
		glBindFramebuffer(GL_FRAMEBUFFER, ShadowFrameBuffer);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glBindFramebuffer(GL_FRAMEBUFFER, viewFbo->getName());
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glBindFramebuffer(GL_FRAMEBUFFER, pingPongFbo[0]->getName());
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glBindFramebuffer(GL_FRAMEBUFFER, pingPongFbo[1]->getName());
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glBindFramebuffer(GL_FRAMEBUFFER, 0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		//--------------------------------------------------------------------------------------------------------------------------------
		// Render in die ShadowMap
		glBindFramebuffer(GL_FRAMEBUFFER, ShadowFrameBuffer);
		glViewport(0, 0, 1024, 1024);
		glEnable(GL_CULL_FACE);
		glCullFace(GL_BACK);
		// Benutze Tiefenshader
		glUseProgram(depthShader->programHandle);

		for (const auto model : sceneModels_Map) {
			glm::mat4 depthVP = depthProjectionMatrix * depthViewMatrix;
			// setze shader Variablen
			glUniformMatrix4fv(depthMatrixID, 1, GL_FALSE, &depthVP[0][0]);
			//Zeichne Object
			model.second->draw(depthModelMatrixID);

		}

		//--------------------------------------------------------------------------------------------------------------------------------
		// Render auf den Bildschirm
		glBindFramebuffer(GL_FRAMEBUFFER, viewFbo->getName());
		//set Texture Sampling Mode and Mip maps
		if (actualSampling == TextureSampling::Linear) {
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			if (actualMipMapping == MipMapping::LinearMip) {
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
			}
			else if (actualMipMapping == MipMapping::NearestMip) {
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
			}
			else {
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			}
		}
		else {
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
			if (actualMipMapping == MipMapping::LinearMip) {
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
			}
			else if (actualMipMapping == MipMapping::NearestMip) {
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
			}
			else {
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			}
		}
		//#########################################
		glViewport(0, 0, width, height);
		glEnable(GL_CULL_FACE);
		glCullFace(GL_BACK);
		// Benutze std. Shader
		glUseProgram(shader->programHandle);
		//Bind ViewMatrix
		glUniformMatrix4fv(ViewMatrixID, 1, GL_FALSE, &ViewMatrix[0][0]);
		// binde shadow Map an pos 1 
		glActiveTexture(GL_TEXTURE1);
		glBindTexture(GL_TEXTURE_2D, depthTexture);
		glUniform1i(ShadowMapID, 1);
		//bind Light
		sunLight->bind(shader->programHandle);
		for (const auto model : sceneModels_Map) {

			// frustum culling
			if (!viewFrustumCullingOn || SphereInFrustum(frustum, model.second->getSphere()))
			{
				internCounter++;

				// Berechne Matritzen
				glm::mat4 VP = ProjectionMatrix * ViewMatrix;
				glm::mat4 depthVP = depthProjectionMatrix * depthViewMatrix;
				glm::mat4 depthBiasVP = biasMatrix*depthVP;
				// Setze shader variablen
				glUniformMatrix4fv(MatrixID, 1, GL_FALSE, &VP[0][0]);
				glUniformMatrix4fv(DepthBiasID, 1, GL_FALSE, &depthBiasVP[0][0]);
				glUniform3f(lightInvDirID, lightInvDir.x, lightInvDir.y, lightInvDir.z);
				// binde die Texture an pos 0
				// tex->bind(shader->programHandle);
				// binde Material TODO in eigene klasse auslagern
				auto baseColor = glGetUniformLocation(shader->programHandle, "mat.baseColor");
				glUniform3f(baseColor, 0.8f, 1, 1);
				auto specColor = glGetUniformLocation(shader->programHandle, "mat.specColor");
				glUniform3f(specColor, 1, 0.8f, 0.8f);
				auto shininess = glGetUniformLocation(shader->programHandle, "mat.specShininess");
				glUniform1f(shininess, 50);
				auto specPower = glGetUniformLocation(shader->programHandle, "mat.specPower");
				glUniform1f(specPower, 0.5f);
				//Zeichne Model
				if (!wireFrame)
				{
					glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
				}
				else
				{
					glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
				}

				model.second->draw(ModelMatrixID);
			}
			
		}
		//Draw Transparent model
		if (transparent) {
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		}
		for (const auto model : itemModels_Map) {

			// frustum culling
			if (!viewFrustumCullingOn || SphereInFrustum(frustum, model.second->getSphere()))
			{

				if (((Item*)model.second)->alive) {

					internCounter++;
					// Berechne Matritzen
					glm::mat4 VP = ProjectionMatrix * ViewMatrix;
					glm::mat4 depthVP = depthProjectionMatrix * depthViewMatrix;
					glm::mat4 depthBiasVP = biasMatrix*depthVP;
					// Setze shader variablen
					glUniformMatrix4fv(MatrixID, 1, GL_FALSE, &VP[0][0]);
					glUniformMatrix4fv(DepthBiasID, 1, GL_FALSE, &depthBiasVP[0][0]);
					glUniform3f(lightInvDirID, lightInvDir.x, lightInvDir.y, lightInvDir.z);
					// binde die Texture an pos 0
					// tex->bind(shader->programHandle);
					// binde Material TODO in eigene klasse auslagern
					auto baseColor = glGetUniformLocation(shader->programHandle, "mat.baseColor");
					glUniform3f(baseColor, 2, 2, 2);
					auto specColor = glGetUniformLocation(shader->programHandle, "mat.specColor");
					glUniform3f(specColor, 1, 1, 1);
					auto shininess = glGetUniformLocation(shader->programHandle, "mat.specShininess");
					glUniform1f(shininess, 5);
					auto specPower = glGetUniformLocation(shader->programHandle, "mat.specPower");
					glUniform1f(specPower, 1);
					//Zeichne Model
					if (!wireFrame)
					{
						glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
					}
					else
					{
						glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
					}
					model.second->draw(ModelMatrixID);
				}
			}
		}
		glDisable(GL_BLEND);
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		//---------------------------------------------

		// render particle system

		// std::cout << (cameraposition.x << " " << cameraposition.y << " " << cameraposition.z << std::endl;

		glm::mat4 VP = ProjectionMatrix * ViewMatrix;

		particleSystem.Render(time_delta, VP, cameraposition);

		// testausgabe ob das rendern funktioniert
		// cout << "The Rendering of the particle system is working" << endl;


		// check OpenGL error
		GLenum err;
		while ((err = glGetError()) != GL_NO_ERROR) {
			cerr << "OpenGL error: " << err << endl;
		}

		
		//##########################################################
		//BLOOM
		GLboolean horizontal = true, first_iteration = true;
		GLuint amount = 10;
		// Benutze std. Shader
		glUseProgram(blurShader->programHandle);
		for (GLuint i = 0; i < amount; i++)
		{
			glBindFramebuffer(GL_FRAMEBUFFER, pingPongFbo[horizontal]->getName());
			glUniform1i(blurHorizontalID, horizontal);
			if (first_iteration) {
				glActiveTexture(GL_TEXTURE0);
				glBindTexture(GL_TEXTURE_2D, viewFbo->getBrightTexture());
				glUniform1i(blurTextureID, 0);
			}
			else  {
				glActiveTexture(GL_TEXTURE0);
				glBindTexture(GL_TEXTURE_2D, pingPongFbo[!horizontal]->getTexture());
				glUniform1i(blurTextureID, 0);
			}
			pingPongFbo[horizontal]->drawOnScreen();
			horizontal = !horizontal;
			if (first_iteration)
				first_iteration = false;
		}
		//##########################################################

		// Render auf den Bildschirm
		glBindFramebuffer(GL_FRAMEBUFFER, 0);
		glViewport(0, 0, width, height);
		// Benutze std. Shader
		glUseProgram(viewShader->programHandle);

		glm::mat4 prevMVP = activeCamera->getProjMatrix() * activeCamera->modelMatrix;
		glm::mat4 invMV = glm::inverse(activeCamera->getProjMatrix() * activeCamera->modelMatrix);
		// Setze shader variablen
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, viewFbo->getTexture());
		glUniform1i(viewTexId, 0);
		glActiveTexture(GL_TEXTURE2);
		glBindTexture(GL_TEXTURE_2D, viewFbo->getDepthTexture());
		glUniform1i(viewDepthTexId, 2);
		glActiveTexture(GL_TEXTURE1);
		glBindTexture(GL_TEXTURE_2D, pingPongFbo[false]->getTexture());
		glUniform1i(bloomTexId, 1);


		glUniformMatrix4fv(invMVID, 1, GL_FALSE, &invMV[0][0]);
		viewFbo->drawOnScreen();
		glUniformMatrix4fv(prevMVPID, 1, GL_FALSE, &prevMVP[0][0]);
	}
	else if (actualGameStatus == GameStatus::WIN) {
		//Draw Win pic
		// Render auf den Bildschirm
		glBindFramebuffer(GL_FRAMEBUFFER, 0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glViewport(0, 0, width, height);
		// Benutze std. Shader
		//glUseProgram(simpleShader->programHandle);
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, textures_Map.find("winTexture")->second->getHandle());
		glUniform1i(viewTexId, 0);
		viewFbo->drawOnScreen();
	}
	else {
		//Draw Lose pic
		// Render auf den Bildschirm
		glBindFramebuffer(GL_FRAMEBUFFER, 0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glViewport(0, 0, width, height);
		// Benutze std. Shader
		//glUseProgram(simpleShader->programHandle);
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, textures_Map.find("loseTexture")->second->getHandle());
		glUniform1i(viewTexId, 0);
		viewFbo->drawOnScreen();
	}
	counter = internCounter;
}

void cleanup() {
	//FBOs

	//Shader
	delete shader; shader = nullptr;
	delete viewShader; viewShader = nullptr;
	delete depthShader; depthShader = nullptr;
	delete simpleShader; simpleShader = nullptr;
	delete blurShader; blurShader = nullptr;

	//SceneObjects
	for (auto model : sceneModels_Map) {
		delete model.second; model.second = nullptr;
	}
	for (auto model : itemModels_Map) {
		delete model.second; model.second = nullptr;
	}

	//Entities
	delete player; player = nullptr;
	
	//Textures
	for (auto tex : textures_Map) {
		delete tex.second; tex.second = nullptr;
	}
	

	//Lights
	delete sunLight; sunLight = nullptr;

	//Cameras
	delete freeCamera; freeCamera = nullptr;
	delete gameCamera; gameCamera = nullptr;
	activeCamera = nullptr;

	//Physics
	physxManager->cleanup();
}

// used for particleEffects with geometry shader
GLuint LoadProgram(const std::string& vs, const std::string& gs, const std::string& fs)
{
	GLchar compilerSpew[256];

	GLuint programHandle = glCreateProgram();

	// load vertex shader
	std::ifstream shaderFileVs(vs);
	std::string codeVs = std::string(
		std::istreambuf_iterator<char>(shaderFileVs),
		std::istreambuf_iterator<char>());
	shaderFileVs.close();
	const char * vsSource = codeVs.c_str();

	GLuint vsHandle = glCreateShader(GL_VERTEX_SHADER);
	glShaderSource(vsHandle, 1, &vsSource, nullptr);
	glCompileShader(vsHandle);
	glGetShaderInfoLog(vsHandle, sizeof(compilerSpew), 0, compilerSpew);
	glAttachShader(programHandle, vsHandle);

	// load fragment shader
	std::ifstream shaderFileFs(fs);
	std::string codeFs = std::string(
		std::istreambuf_iterator<char>(shaderFileFs),
		std::istreambuf_iterator<char>());
	shaderFileFs.close();
	const char * fsSource = codeFs.c_str();

	GLuint fsHandle = glCreateShader(GL_FRAGMENT_SHADER);
	glShaderSource(fsHandle, 1, &fsSource, nullptr);
	glCompileShader(fsHandle);
	glGetShaderInfoLog(fsHandle, sizeof(compilerSpew), 0, compilerSpew);
	glAttachShader(programHandle, fsHandle);

	// load geometry shader
	std::ifstream shaderFileGs(gs);
	std::string codeGs = std::string(
		std::istreambuf_iterator<char>(shaderFileGs),
		std::istreambuf_iterator<char>());
	shaderFileGs.close();
	const char * gsSource = codeGs.c_str();

	GLuint gsHandle = glCreateShader(GL_GEOMETRY_SHADER);
	glShaderSource(gsHandle, 1, &gsSource, nullptr);
	glCompileShader(gsHandle);
	glGetShaderInfoLog(gsHandle, sizeof(compilerSpew), 0, compilerSpew);
	glAttachShader(programHandle, gsHandle);

	glBindAttribLocation(programHandle, SlotPosition, "Position");
	glBindAttribLocation(programHandle, SlotTexCoord, "TexCoord");
	glBindAttribLocation(programHandle, SlotNormal, "Normal");
	glBindAttribLocation(programHandle, SlotBirthTime, "BirthTime");
	glBindAttribLocation(programHandle, SlotVelocity, "Velocity");

	/*
	if (!pF) {
		const char* varyings[3] = { "vPosition", "vBirthTime", "vVelocity" };
		glTransformFeedbackVaryings(programHandle, 3, varyings, GL_INTERLEAVED_ATTRIBS);
	}
	*/

	glLinkProgram(programHandle);
	glGetProgramInfoLog(programHandle, sizeof(compilerSpew), 0, compilerSpew);

	glUseProgram(programHandle);
	return programHandle;

}

// used for particleEffects for advection shader and the particlevertexshader without fragment
GLuint LoadProgram(const std::string& vs)
{
	GLchar compilerSpew[256];

	GLuint programHandle = glCreateProgram();

	// load vertex shader
	std::ifstream shaderFileVs(vs);
	std::string codeVs = std::string(
		std::istreambuf_iterator<char>(shaderFileVs),
		std::istreambuf_iterator<char>());
	shaderFileVs.close();
	const char * vsSource = codeVs.c_str();

	GLuint vsHandle = glCreateShader(GL_VERTEX_SHADER);
	glShaderSource(vsHandle, 1, &vsSource, nullptr);
	glCompileShader(vsHandle);
	glGetShaderInfoLog(vsHandle, sizeof(compilerSpew), 0, compilerSpew);
	glAttachShader(programHandle, vsHandle);

	glBindAttribLocation(programHandle, SlotPosition, "Position");
	glBindAttribLocation(programHandle, SlotTexCoord, "TexCoord");
	glBindAttribLocation(programHandle, SlotNormal, "Normal");
	glBindAttribLocation(programHandle, SlotBirthTime, "BirthTime");
	glBindAttribLocation(programHandle, SlotVelocity, "Velocity");

	const char* varyings[3] = { "vPosition", "vBirthTime", "vVelocity" };
	glTransformFeedbackVaryings(ParticleProgram, 3, varyings, GL_INTERLEAVED_ATTRIBS);

	glLinkProgram(programHandle);
	glGetProgramInfoLog(programHandle, sizeof(compilerSpew), 0, compilerSpew);

	glUseProgram(programHandle);
	return programHandle;

}





